"""Template tags for the socialprofile module"""

import re

from django import template
from django.template.defaultfilters import stringfilter
from social_core.backends.oauth import OAuthAuth

register = template.Library()

name_re = re.compile(r"([^O])Auth")


@register.filter
@stringfilter
def social_provider_name(provider_slug):
    """Decode name of Social Auth Provider to Friendly Name"""
    if provider_slug == "google-oauth2":
        return "Google"
    if provider_slug == "twitter":
        return "Twitter"
    if provider_slug == "facebook":
        return "Facebook"
    if provider_slug == "live":
        return "Microsoft"
    if provider_slug == "instagram":
        return "Instagram"


@register.filter
@stringfilter
def social_provider_color(provider_slug):
    """Decode name of Social Auth Provider to font-awesome btn-{color}"""
    if provider_slug == "google-oauth2":
        return "red"
    if provider_slug == "twitter":
        return "aqua"
    if provider_slug == "facebook":
        return "blue"
    if provider_slug == "instagram":
        return "salmon"
    if provider_slug == "live":
        return "forest"
    if provider_slug == "apple":
        return "navy"


@register.filter
def backend_name(backend):
    name = backend.__class__.__name__
    name = name.replace("OAuth", " OAuth")
    name = name.replace("OpenId", " OpenId")
    name = name.replace("Sandbox", "")
    name = name_re.sub(r"\1 Auth", name)
    return name


@register.filter
def backend_class(backend):
    return backend.name.replace("-", " ")


@register.filter
def icon_name(name):
    return {
        "stackoverflow": "stack-overflow",
        "google-oauth": "google",
        "google-oauth2": "google",
        "google-openidconnect": "google",
        "dropbox-oauth2": "dropbox",
        "yahoo-oauth": "yahoo",
        "facebook-app": "facebook",
        "email": "envelope",
        "vimeo": "vimeo-square",
        "linkedin-oauth2": "linkedin",
        "vk-oauth2": "vk",
        "live": "windows",
        "username": "user",
    }.get(name, name)


@register.filter
def social_backends(backends):
    backends = [
        (name, backend)
        for name, backend in backends.items()
        if name not in ["username", "email"]
    ]
    backends.sort(key=lambda b: b[0])
    return [backends[n : n + 10] for n in range(0, len(backends), 10)]


@register.filter
def legacy_backends(backends):
    backends = [
        (name, backend)
        for name, backend in backends.items()
        if name in ["username", "email"]
    ]
    backends.sort(key=lambda b: b[0])
    return backends


@register.filter
def oauth_backends(backends):
    backends = [
        (name, backend)
        for name, backend in backends.items()
        if issubclass(backend, OAuthAuth)
    ]
    backends.sort(key=lambda b: b[0])
    return backends


@register.simple_tag(takes_context=True)
def associated(context, backend):
    user = context.get("user")
    context["association"] = None
    if user and user.is_authenticated:
        try:
            context["association"] = user.social_auth.filter(provider=backend.name)[0]
        except IndexError:
            pass
    return ""



"""Python Social Auth Pipeline Extensions"""

from requests import request
from social_core.backends.facebook import FacebookOAuth2
from social_core.backends.google import GoogleOAuth2
from social_core.backends.live import LiveOAuth2
from social_core.backends.twitter import TwitterOAuth

# def create_profile(user, is_new=False, *args, **kwargs):
#     """Create a profile instance for the given user"""
#     if is_new:
#         profile = user.get_profile()
#         if profile is None:
#             profile = Profile(user_id=user.id)
#         profile.gender = response.get("gender")
#         profile.link = response.get("link")
#         profile.timezone = response.get("timezone")
#         profile.save()


def socialprofile_extra_values(backend, details, response, uid, user, *args, **kwargs):
    """Routes the extra values call to the appropriate back end"""
    if type(backend) is GoogleOAuth2:
        return {
            "user": google_extra_values(
                backend, details, response, uid, user, *args, **kwargs
            )
        }

    if type(backend) is TwitterOAuth:
        return {
            "user": twitter_extra_values(
                backend, details, response, uid, user, *args, **kwargs
            )
        }

    if type(backend) is FacebookOAuth2:
        return {
            "user": facebook_extra_values(
                backend, details, response, uid, user, *args, **kwargs
            )
        }

    if type(backend) is LiveOAuth2:
        return {
            "user": live_extra_values(
                backend, details, response, uid, user, *args, **kwargs
            )
        }


def google_extra_values(backend, details, response, uid, user, *args, **kwargs):
    """Populates a UserProfile Object when a new User is created via Google Auth"""
    # ToDo: should be placed in new pipeline before
    # if kwargs.get("new_association"):
    #     print("is new google association!")
    #     try:
    #         req_response = request(
    #             "GET",
    #             "https://www.googleapis.com/oauth2/v3/userinfo",
    #             headers={
    #                 "Authorization": f"Bearer {kwargs.get('access_token')}",
    #             },
    #         ).json()
    #         print(req_response)
    #     except Exception as err:
    #         print(err)

    if not user.edited_by_user:
        user.last_name = response.get("family_name", "")
        user.first_name = response.get("given_name", "")
        user.save()

        date_of_birth = response.get("birthday")
        if date_of_birth:
            user.date_of_birth = date_of_birth
        gender = response.get("gender")
        if gender:
            user.gender = gender
        country = response.get("locale")
        if country:
            user.country = country
        description = response.get("occupation")
        if description:
            user.description = description

    google_username = response.get("username")
    if google_username:
        user.google_username = google_username

    user.google_verified = response.get("email_verified", False)
    user.google_isPlusUser = response.get("isPlusUser", False)
    user.google_url = response.get("url", "")
    user.google_circledByCount = response.get("circledByCount", 0)
    user.google_language = response.get("language", "")
    user.google_kind = response.get("kind", "")
    user.google_avatar = response.get("picture", "")

    user.edited_by_google = True
    user.save()
    return user


def facebook_extra_values(backend, details, response, uid, user, *args, **kwargs):
    """Populates a UserProfile Object when a new User is created via Facebook Auth"""
    if not user.edited_by_user:
        user.last_name = response.get("last_name", "")
        user.first_name = response.get("first_name", "")
        user.save()

        gender = response.get("gender")
        if gender:
            user.gender = gender

    username_facebook = response.get("username")
    if username_facebook:
        user.username_facebook = username_facebook
    profile_url = response.get("link")
    if profile_url:
        user.facebook_url = profile_url
    if response.get("picture", False):
        user.facebook_avatar = response.get("picture").get("data").get("url", "")
    user.edited_by_facebook = True
    user.save()
    return user


def twitter_extra_values(backend, details, response, uid, user, *args, **kwargs):
    """Populates a UserProfile Object when a new User is created via Twitter Auth"""
    if not user.edited_by_user:
        try:
            first_name, last_name = response.get("name", "").split(" ", 1)
        except Exception:
            first_name = response.get("name", "")
            last_name = ""
        user.last_name = last_name
        user.first_name = first_name
        user.save()

        description = response.get("description")
        if description:
            user.description = description

    username_twitter = response.get("username")
    if username_twitter:
        user.username_twitter = username_twitter
    profile_url = response.get("url")
    if profile_url:
        user.twitter_url = profile_url
    user.twitter_language = response.get("lang", "")
    user.twitter_verified = response.get("verified", False)
    user.twitter_avatar = response.get("profile_image_url_https", "")
    user.edited_by_twitter = True

    user.save()
    return user


def live_extra_values(backend, details, response, uid, user, *args, **kwargs):
    """Populates a UserProfile Object when a new User is created via Live Auth"""
    if not user.edited_by_user:
        user.last_name = response.get("last_name", "")
        user.first_name = response.get("first_name", "")
        user.save()

        gender = response.get("gender")
        if gender:
            user.gender = gender

    username_live = response.get("username")
    if username_live:
        user.username_live = username_live
    profile_url = response.get("link")
    if profile_url:
        user.live_url = profile_url
    user.live_language = response.get("locale", "")

    user.edited_by_live = True
    user.save()
    return user



# Generated by Django 2.2.19 on 2021-10-30 22:37

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("socialprofile", "0004_auto_20200305_1946"),
    ]

    operations = [
        migrations.AlterField(
            model_name="socialprofile",
            name="google_kind",
            field=models.CharField(
                blank=True, max_length=15, null=True, verbose_name="Google Kind"
            ),
        ),
    ]



# Generated by Django 3.2.13 on 2022-05-03 21:58

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("socialprofile", "0026_socialprofile_edited_by_live"),
    ]

    operations = [
        # migrations.DeleteModel(
        #     name="ProxyPhoneDevice",
        # ),
    ]



# Generated by Django 2.2.24 on 2021-11-01 11:14

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("socialprofile", "0009_auto_20210214_1722"),
    ]

    operations = [
        migrations.AlterField(
            model_name="socialprofile",
            name="twitter_language",
            field=models.CharField(
                blank=True, max_length=10, null=True, verbose_name="Twitter Language"
            ),
        ),
    ]



"""Unit Tests for the socialprofile module user"""

# pylint: disable=R0904, C0103

from unittest import skipIf, skipUnless
from unittest.mock import patch

import django
from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.auth.middleware import AuthenticationMiddleware
from django.core import mail, management
from django.db import connection
from django.http import HttpRequest
from django.test import TestCase
from django.utils import timezone

try:
    from django.utils.encoding import force_text
except ImportError:
    # Only available from Django 4, ignore the tests otherwise
    from django.utils.encoding import force_str

# from socialprofile.forms import EmailUserChangeForm, EmailUserCreationForm

try:
    from django.contrib.auth.middleware import SessionAuthenticationMiddleware
except ImportError:
    # Only available from Django 1.7, ignore the tests otherwise
    SessionAuthenticationMiddleware = None


class UserTest(TestCase):
    user_email = "newuser@localhost.local"
    user_password = "1234"

    def create_user(self):
        """Create and return a new user with self.user_email as login and self.user_password as password."""
        return get_user_model().objects.create_user(self.user_email, self.user_password)

    def test_user_creation(self):
        # Create a new user saving the time frame
        right_now = timezone.now().replace(
            microsecond=0
        )  # MySQL doesn't store microseconds
        with patch.object(timezone, "now", return_value=right_now):
            self.create_user()

        # Check user exists and email is correct
        self.assertEqual(get_user_model().objects.all().count(), 1)
        self.assertEqual(get_user_model().objects.all()[0].email, self.user_email)

        # Check date_joined and last_login dates
        self.assertEqual(get_user_model().objects.all()[0].date_joined, right_now)
        self.assertEqual(get_user_model().objects.all()[0].last_login, right_now)

        # Check flags
        self.assertTrue(get_user_model().objects.all()[0].is_active)
        self.assertFalse(get_user_model().objects.all()[0].is_staff)
        self.assertFalse(get_user_model().objects.all()[0].is_superuser)

    def test_user_get_full_name(self):
        user = self.create_user()
        self.assertEqual(user.get_full_name(), self.user_email)

    def test_user_get_short_name(self):
        user = self.create_user()
        self.assertEqual(user.get_short_name(), self.user_email)

    def test_email_user(self):
        # Email definition
        subject = "Email Subject"
        message = "Email Message"
        from_email = "from@normal.com"

        user = self.create_user()

        # Test that no message exists
        self.assertEqual(len(mail.outbox), 0)

        # Send test email
        user.email_user(subject, message, from_email)

        # Test that one message has been sent
        self.assertEqual(len(mail.outbox), 1)

        # Verify that the email is correct
        self.assertEqual(mail.outbox[0].subject, subject)
        self.assertEqual(mail.outbox[0].body, message)
        self.assertEqual(mail.outbox[0].from_email, from_email)
        self.assertEqual(mail.outbox[0].to, [user.email])

    def test_email_user_kwargs(self):
        # valid send_mail parameters
        kwargs = {
            "fail_silently": False,
            "auth_user": None,
            "auth_password": None,
            "connection": None,
        }
        user = get_user_model()(email="foo@bar.com")
        user.email_user(
            subject="Subject here",
            message="This is a message",
            from_email="from@domain.com",
            **kwargs,
        )
        # Test that one message has been sent.
        self.assertEqual(len(mail.outbox), 1)
        # Verify that test email contains the correct attributes:
        message = mail.outbox[0]
        self.assertEqual(message.subject, "Subject here")
        self.assertEqual(message.body, "This is a message")
        self.assertEqual(message.from_email, "from@domain.com")
        self.assertEqual(message.to, [user.email])


class UserManagerTest(TestCase):
    def test_create_user(self):
        email_lowercase = "normal@user.test"
        user = get_user_model().objects.create_user(email_lowercase)
        self.assertEqual(user.email, email_lowercase)
        self.assertFalse(user.has_usable_password())
        self.assertTrue(user.is_active)
        self.assertFalse(user.is_staff)
        self.assertFalse(user.is_superuser)

    def test_create_superuser(self):
        email_lowercase = "super@user.test"
        password = "password1234$%&/"
        user = get_user_model().objects.create_superuser(email_lowercase, password)
        self.assertEqual(user.email, email_lowercase)
        self.assertTrue(user.check_password, password)
        self.assertTrue(user.is_active)
        self.assertTrue(user.is_staff)
        self.assertTrue(user.is_superuser)

    def test_user_creation_is_active(self):
        # Create deactivated user
        email_lowercase = "disabled@user.test"
        password = "password1234$%&/"
        user = get_user_model().objects.create_user(
            email_lowercase, password, is_active=False
        )
        self.assertFalse(user.is_active)

    def test_user_creation_is_staff(self):
        # Create staff user
        email_lowercase = "staff@user.test"
        password = "password1234$%&/"
        user = get_user_model().objects.create_user(
            email_lowercase, password, is_staff=True
        )
        self.assertTrue(user.is_staff)

    def test_create_user_email_domain_normalize_rfc3696(self):
        # According to http://tools.ietf.org/html/rfc3696#section-3
        # the "@" symbol can be part of the local part of an email address
        returned = get_user_model().objects.normalize_email(r"Abc\@DEF@EXAMPLE.com")
        self.assertEqual(returned, r"Abc\@DEF@example.com")

    def test_create_user_email_domain_normalize(self):
        returned = get_user_model().objects.normalize_email("normal@DOMAIN.COM")
        self.assertEqual(returned, "normal@domain.com")

    def test_create_user_email_domain_normalize_with_whitespace(self):
        returned = get_user_model().objects.normalize_email(
            r"email\ with_whitespace@D.COM"
        )
        self.assertEqual(returned, r"email\ with_whitespace@d.com")

    def test_empty_username(self):
        self.assertRaisesMessage(
            ValueError,
            "The given email must be set",
            get_user_model().objects.create_user,
            email="",
        )


@skipIf(django.VERSION < (1, 7, 0), "Migrations not available in this Django version")
class MigrationsTest(TestCase):
    def test_makemigrations_no_changes(self):
        try:
            from StringIO import StringIO
        except ImportError:
            from io import StringIO

        with patch("sys.stdout", new_callable=StringIO) as mock:
            management.call_command("makemigrations", "socialprofile", dry_run=True)
        self.assertEqual(
            mock.getvalue(), "No changes detected in app 'socialprofile'\n"
        )

    @skipUnless(django.VERSION[:2] == (1, 7), "Monkey patch only applied to Django 1.7")
    def test_monkey_patch_model_field(self):
        field_last_login = get_user_model()._meta.get_field("last_login")
        self.assertTrue(field_last_login.null)
        self.assertTrue(field_last_login.blank)

    @skipUnless(django.VERSION[:2] == (1, 7), "Monkey patch only applied to Django 1.7")
    def test_monkey_patch_db_column(self):
        table_name = get_user_model()._meta.db_table
        cursor = connection.cursor()
        table_fields = connection.introspection.get_table_description(
            cursor, table_name
        )
        field = next(
            field for field in table_fields if field.name == "last_login"
        )  # pragma: no cover
        self.assertTrue(field.null_ok)

    @skipUnless(django.VERSION[:2] == (1, 7), "Monkey patch only applied to Django 1.7")
    def test_monkey_patch_side_effects(self):
        # Check the parent model isn't affected from the monkey patch
        from django.contrib.auth.models import AbstractBaseUser

        field_last_login = AbstractBaseUser._meta.get_field("last_login")
        self.assertFalse(field_last_login.null)
        self.assertFalse(field_last_login.blank)


@skipIf(
    SessionAuthenticationMiddleware is None,
    "SessionAuthenticationMiddleware not available in this version",
)
class TestSessionAuthenticationMiddleware(TestCase):
    def setUp(self):
        self.user_email = "test@example.com"
        self.user_password = "test_password"
        self.user_username = "test_username"
        self.user = get_user_model().objects.create_user(
            self.user_email, self.user_password, self.user_username
        )

        self.middleware_auth = AuthenticationMiddleware()
        self.middleware_session_auth = SessionAuthenticationMiddleware()
        self.assertTrue(
            self.client.login(
                username=self.user_email,
                password=self.user_password,
            )
        )
        self.request = HttpRequest()
        self.request.session = self.client.session

    def test_changed_password_doesnt_invalidate_session(self):
        # Changing a user's password shouldn't invalidate the session if session
        # verification isn't activated.
        session_key = self.request.session.session_key
        self.middleware_auth.process_request(self.request)
        self.middleware_session_auth.process_request(self.request)
        self.assertIsNotNone(self.request.user)
        self.assertFalse(self.request.user.is_anonymous())

        # After password change, user should remain logged in.
        self.user.set_password("new_password")
        self.user.save()
        self.middleware_auth.process_request(self.request)
        self.middleware_session_auth.process_request(self.request)
        self.assertIsNotNone(self.request.user)
        self.assertFalse(self.request.user.is_anonymous())
        self.assertEqual(session_key, self.request.session.session_key)

    def test_changed_password_invalidates_session_with_middleware(self):
        session_key = self.request.session.session_key
        with self.modify_settings(
            MIDDLEWARE_CLASSES={
                "append": [
                    "django.contrib.auth.middleware.SessionAuthenticationMiddleware"
                ]
            }
        ):
            # After password change, user should be anonymous
            self.user.set_password("new_password")
            self.user.save()
            self.middleware_auth.process_request(self.request)
            self.middleware_session_auth.process_request(self.request)
            self.assertIsNotNone(self.request.user)
            self.assertTrue(self.request.user.is_anonymous())
        # session should be flushed
        self.assertNotEqual(session_key, self.request.session.session_key)


# @override_settings(
#     USE_TZ=False, PASSWORD_HASHERS=("django.contrib.auth.hashers.SHA1PasswordHasher",)
# )
# class EmailUserCreationFormTest(TestCase):
#     def setUp(self):
#         get_user_model().objects.create_user("testclient@example.com", "test123")
#
#     def test_user_already_exists(self):
#         data = {
#             "email": "testclient@example.com",
#             "password1": "test123",
#             "password2": "test123",
#         }
#         form = EmailUserCreationForm(data)
#         self.assertFalse(form.is_valid())
#         self.assertEqual(
#             form["email"].errors, [force_text(form.error_messages["duplicate_email"])]
#         )
#
#     def test_invalid_data(self):
#         data = {
#             "email": "testclient",
#             "password1": "test123",
#             "password2": "test123",
#         }
#         form = EmailUserCreationForm(data)
#         self.assertFalse(form.is_valid())
#         self.assertEqual(form["email"].errors, [_("Enter a valid email address.")])
#
#     def test_password_verification(self):
#         # The verification password is incorrect.
#         data = {
#             "email": "testclient@example.com",
#             "password1": "test123",
#             "password2": "test",
#         }
#         form = EmailUserCreationForm(data)
#         self.assertFalse(form.is_valid())
#         self.assertEqual(
#             form["password2"].errors,
#             [force_text(form.error_messages["password_mismatch"])],
#         )
#
#     def test_both_passwords(self):
#         # One (or both) passwords weren't given
#         data = {"email": "testclient@example.com"}
#         form = EmailUserCreationForm(data)
#         required_error = [force_text(Field.default_error_messages["required"])]
#         self.assertFalse(form.is_valid())
#         self.assertEqual(form["password1"].errors, required_error)
#         self.assertEqual(form["password2"].errors, required_error)
#
#         data["password2"] = "test123"
#         form = EmailUserCreationForm(data)
#         self.assertFalse(form.is_valid())
#         self.assertEqual(form["password1"].errors, required_error)
#         self.assertEqual(form["password2"].errors, [])
#
#     def test_success(self):
#         # The success case.
#         data = {
#             "email": "jsmith@example.com",
#             "password1": "test123",
#             "password2": "test123",
#         }
#         form = EmailUserCreationForm(data)
#         self.assertTrue(form.is_valid())
#         u = form.save()
#         self.assertEqual(
#             repr(u), "<%s: jsmith@example.com>" % get_user_model().__name__
#         )
#         self.assertIsNotNone(u.pk)
#
#     def test_success_without_commit(self):
#         # The success case, but without saving the user instance to the db.
#         data = {
#             "email": "jsmith@example.com",
#             "password1": "test123",
#             "password2": "test123",
#         }
#         form = EmailUserCreationForm(data)
#         self.assertTrue(form.is_valid())
#         u = form.save(commit=False)
#         self.assertEqual(
#             repr(u), "<%s: jsmith@example.com>" % get_user_model().__name__
#         )
#         self.assertIsNone(u.pk, None)


# @override_settings(
#     USE_TZ=False, PASSWORD_HASHERS=("django.contrib.auth.hashers.SHA1PasswordHasher",)
# )
# class EmailUserChangeFormTest(TestCase):
#     def setUp(self):
#         user = get_user_model().objects.create_user("testclient@example.com")
#         user.password = "sha1$6efc0$f93efe9fd7542f25a7be94871ea45aa95de57161"
#         user.save()
#         get_user_model().objects.create_user("empty_password@example.com")
#         user_unmanageable = get_user_model().objects.create_user(
#             "unmanageable_password@example.com"
#         )
#         user_unmanageable.password = "$"
#         user_unmanageable.save()
#         user_unknown = get_user_model().objects.create_user(
#             "unknown_password@example.com"
#         )
#         user_unknown.password = "foo$bar"
#         user_unknown.save()
#
#     def test_username_validity(self):
#         user = get_user_model().objects.get(email="testclient@example.com")
#         data = {"email": "not valid"}
#         form = EmailUserChangeForm(data, instance=user)
#         self.assertFalse(form.is_valid())
#         self.assertEqual(form["email"].errors, [_("Enter a valid email address.")])
#
#     def test_bug_14242(self):
#         # A regression test, introduce by adding an optimization for the
#         # EmailUserChangeForm.
#
#         class MyUserForm(EmailUserChangeForm):
#             def __init__(self, *args, **kwargs):
#                 super().__init__(*args, **kwargs)
#                 self.fields[
#                     "groups"
#                 ].help_text = "These groups give users different permissions"
#
#             class Meta(EmailUserChangeForm.Meta):
#                 fields = ("groups",)
#
#         # Just check we can create it
#         MyUserForm({})
#
#     def test_unsuable_password(self):
#         user = get_user_model().objects.get(email="empty_password@example.com")
#         user.set_unusable_password()
#         user.save()
#         form = EmailUserChangeForm(instance=user)
#         self.assertIn(_("No password set."), form.as_table())
#
#     def test_bug_17944_empty_password(self):
#         user = get_user_model().objects.get(email="empty_password@example.com")
#         form = EmailUserChangeForm(instance=user)
#         self.assertIn(_("No password set."), form.as_table())
#
#     def test_bug_17944_unmanageable_password(self):
#         user = get_user_model().objects.get(email="unmanageable_password@example.com")
#         form = EmailUserChangeForm(instance=user)
#         self.assertIn(
#             _("Invalid password format or unknown hashing algorithm."), form.as_table()
#         )
#
#     def test_bug_17944_unknown_password_algorithm(self):
#         user = get_user_model().objects.get(email="unknown_password@example.com")
#         form = EmailUserChangeForm(instance=user)
#         self.assertIn(
#             _("Invalid password format or unknown hashing algorithm."), form.as_table()
#         )
#
#     def test_bug_19133(self):
#         """The change form does not return the password value."""
#         # Use the form to construct the POST data
#         user = get_user_model().objects.get(email="testclient@example.com")
#         form_for_data = EmailUserChangeForm(instance=user)
#         post_data = form_for_data.initial
#
#         # The password field should be readonly, so anything
#         # posted here should be ignored; the form will be
#         # valid, and give back the 'initial' value for the
#         # password field.
#         post_data["password"] = "new password"
#         form = EmailUserChangeForm(instance=user, data=post_data)
#
#         self.assertTrue(form.is_valid())
#         self.assertEqual(
#             form.cleaned_data["password"],
#             "sha1$6efc0$f93efe9fd7542f25a7be94871ea45aa95de57161",
#         )
#
#     def test_bug_19349_bound_password_field(self):
#         user = get_user_model().objects.get(email="testclient@example.com")
#         form = EmailUserChangeForm(data={}, instance=user)
#         # When rendering the bound password field,
#         # ReadOnlyPasswordHashWidget needs the initial
#         # value to render correctly
#         self.assertEqual(form.initial["password"], form["password"].value())


class EmailUserAdminTest(TestCase):
    def setUp(self):
        self.user_email = "test@example.com"
        self.user_password = "test_password"
        self.user_username = "test_username"
        self.user = get_user_model().objects.create_user(
            self.user_email, self.user_password, self.user_username
        )
        self.user.set_password(self.user_password)

        if settings.AUTH_USER_MODEL == "custom_user.EmailUser":
            self.app_name = "custom_user"
            self.model_name = "emailuser"
            self.model_verbose_name = "user"
            self.model_verbose_name_plural = "Users"
            if django.VERSION[:2] < (1, 7):
                self.app_verbose_name = "Custom_User"
            else:
                self.app_verbose_name = "Custom User"
        if settings.AUTH_USER_MODEL == "test_custom_user_subclass.MyCustomEmailUser":
            self.app_name = "test_custom_user_subclass"
            self.model_name = "mycustomemailuser"
            self.model_verbose_name = "MyCustomEmailUserVerboseName"
            self.model_verbose_name_plural = "MyCustomEmailUserVerboseNamePlural"
            if django.VERSION[:2] < (1, 7):
                self.app_verbose_name = "Test_Custom_User_Subclass"
            else:
                self.app_verbose_name = "Test Custom User Subclass"

    # def test_url(self):
    #     self.assertTrue(
    #         self.client.login(
    #             username=self.user_email,
    #             password=self.user_password,
    #         )
    #     )
    #     response = self.client.get(reverse("admin:app_list", args=(self.app_name,)))
    #     self.assertEqual(response.status_code, 200)
    #
    # def test_app_name(self):
    #     self.assertTrue(
    #         self.client.login(
    #             username=self.user_email,
    #             password=self.user_password,
    #         )
    #     )
    #
    #     response = self.client.get(reverse("admin:app_list", args=(self.app_name,)))
    #     self.assertEqual(response.context["app_list"][0]["name"], self.app_verbose_name)
    #
    # def test_model_name(self):
    #     self.assertTrue(
    #         self.client.login(
    #             username=self.user_email,
    #             password=self.user_password,
    #         )
    #     )
    #
    #     response = self.client.get(
    #         reverse(f"admin:{self.app_name}_{self.model_name}_changelist")
    #     )
    #     self.assertEqual(
    #         force_text(response.context["title"]),
    #         "Select %s to change" % self.model_verbose_name,
    #     )
    #
    # def test_model_name_plural(self):
    #     self.assertTrue(
    #         self.client.login(
    #             username=self.user_email,
    #             password=self.user_password,
    #         )
    #     )
    #
    #     response = self.client.get(reverse("admin:app_list", args=(self.app_name,)))
    #     self.assertEqual(
    #         force_text(response.context["app_list"][0]["models"][0]["name"]),
    #         self.model_verbose_name_plural,
    #     )
    #
    # def test_user_change_password(self):
    #     self.assertTrue(
    #         self.client.login(
    #             username=self.user_email,
    #             password=self.user_password,
    #         )
    #     )
    #
    #     user_change_url = reverse(
    #         f"admin:{self.app_name}_{self.model_name}_change",
    #         args=(self.user.pk,),
    #     )
    #
    #     if django.VERSION[:2] < (1, 8):
    #         # Since reverse() does not exist yet, create path manually
    #         password_change_url = user_change_url + "password/"
    #     else:
    #         password_change_url = reverse(
    #             "admin:auth_user_password_change", args=(self.user.pk,)
    #         )
    #
    #     response = self.client.get(user_change_url)
    #     # Test the link inside password field help_text.
    #     rel_link = re.search(
    #         r'you can change the password using <a href="([^"]*)">this form</a>',
    #         force_text(response.content),
    #     ).groups()[0]
    #     self.assertEqual(
    #         os.path.normpath(user_change_url + rel_link),
    #         os.path.normpath(password_change_url),
    #     )
    #
    #     # Test url is correct.
    #     self.assertEqual(
    #         self.client.get(password_change_url).status_code,
    #         200,
    #     )



# Generated by Django 2.2.25 on 2021-12-15 22:57

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("socialprofile", "0021_auto_20211214_2019"),
    ]

    operations = [
        migrations.AddField(
            model_name="socialprofile",
            name="image_avatar",
            field=models.CharField(
                blank=None,
                choices=[
                    ("socials", "Socials"),
                    ("predef", "Predefined"),
                    ("custom", "Custom"),
                ],
                default="socials",
                max_length=100,
                verbose_name="Avatar Picture",
            ),
        ),
        migrations.AddField(
            model_name="socialprofile",
            name="image_avatar_predef",
            field=models.CharField(
                blank=None,
                choices=[
                    ("avatar1.png", "Male 1"),
                    ("avatar2.png", "Male 2"),
                    ("avatar3.png", "Female 1"),
                    ("avatar4.png", "Male 3"),
                    ("avatar5.png", "Male 4"),
                    ("avatar6.png", "Male 5"),
                    ("avatar7.png", "Male 6"),
                    ("avatar8.png", "Female 2"),
                ],
                default="avatar1.png",
                max_length=100,
                verbose_name="Predef Avatar Picture",
            ),
        ),
    ]



# Generated by Django 2.2.9 on 2020-02-07 10:47

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("socialprofile", "0002_models_proxy"),
        ("socialprofile", "0002_auto_20170107_0237"),
    ]

    operations = []



# Generated by Django 2.2.24 on 2021-11-29 15:39

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("socialprofile", "0012_auto_20211129_1613"),
    ]

    operations = [
        migrations.AlterField(
            model_name="socialprofile",
            name="gender",
            field=models.CharField(
                blank=True,
                choices=[
                    ("male", "Male"),
                    ("female", "Female"),
                    ("unknown", "Unknown"),
                ],
                default="unknown",
                max_length=10,
                null=True,
                verbose_name="Gender",
            ),
        ),
    ]



# Generated by Django 2.2.11 on 2020-03-05 18:46

import django.core.validators
import image_cropping.fields
from django.db import migrations, models

import socialprofile.models


class Migration(migrations.Migration):
    dependencies = [
        ("socialprofile", "0003_merge_20200207_1147"),
    ]

    operations = [
        migrations.AddField(
            model_name="socialprofile",
            name="address",
            field=models.CharField(
                blank=True, max_length=255, null=True, verbose_name="Address"
            ),
        ),
        migrations.AddField(
            model_name="socialprofile",
            name="city",
            field=models.CharField(
                blank=True, max_length=255, null=True, verbose_name="City"
            ),
        ),
        migrations.AddField(
            model_name="socialprofile",
            name="company",
            field=models.CharField(
                blank=True, max_length=255, null=True, verbose_name="Company"
            ),
        ),
        migrations.AddField(
            model_name="socialprofile",
            name="last_accessed",
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="socialprofile",
            name="postalcode",
            field=models.CharField(
                blank=True, max_length=255, null=True, verbose_name="Postal Code"
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="country",
            field=models.CharField(
                blank=True, max_length=255, null=True, verbose_name="Country"
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="date_of_birth",
            field=models.DateField(blank=True, null=True, verbose_name="Date of Birth"),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="description",
            field=models.TextField(
                blank=True,
                help_text="Tell us about yourself!",
                null=True,
                verbose_name="Description",
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="editByGoogle",
            field=models.NullBooleanField(
                default=False, verbose_name="Google Edit User's Info"
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="email",
            field=models.EmailField(max_length=254, verbose_name="Email Address"),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="function_01",
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="function_02",
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="function_03",
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="function_04",
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="function_05",
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="function_06",
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="function_07",
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="function_08",
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="function_09",
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="function_10",
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="gender",
            field=models.CharField(
                blank=True,
                choices=[
                    ("male", "Male"),
                    ("female", "Female"),
                    ("other", "Other"),
                    ("", ""),
                ],
                max_length=10,
                null=True,
                verbose_name="Gender",
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="google_circledByCount",
            field=models.IntegerField(
                blank=True, default=0, null=True, verbose_name="Google Circle byCount"
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="google_isPlusUser",
            field=models.NullBooleanField(default=False, verbose_name="Google Plus"),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="google_kind",
            field=models.CharField(
                blank=True, max_length=2, null=True, verbose_name="Google Kind"
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="google_language",
            field=models.CharField(
                blank=True, max_length=2, null=True, verbose_name="Google Language"
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="google_plusUrl",
            field=models.URLField(
                blank=True, max_length=500, null=True, verbose_name="Google Plus Page"
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="google_verified",
            field=models.NullBooleanField(
                default=False, verbose_name="Google Verified"
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="image",
            field=image_cropping.fields.ImageCropField(
                blank=True, null=True, upload_to="user/"
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="image_url",
            field=models.URLField(
                blank=True, max_length=500, null=True, verbose_name="Avatar Picture"
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="is_active",
            field=models.NullBooleanField(
                default=True,
                help_text="Designates whether this user should be treated as active. Unselect this instead of deleting accounts.",
                verbose_name="active",
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="is_staff",
            field=models.NullBooleanField(
                default=False,
                help_text="Designates whether the user can log into this admin site.",
                verbose_name="staff status",
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="manually_edited",
            field=models.NullBooleanField(
                default=False, verbose_name="Manually Edited"
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="phone_number",
            field=models.CharField(
                blank=True,
                max_length=16,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        message="Phone number must be entered in the format: '+999999999'. Up to 15 digits allowed.",
                        regex="^\\+\\d{8,15}$",
                    )
                ],
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="role",
            field=models.CharField(
                blank=True, max_length=500, null=True, verbose_name="Role"
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="sort",
            field=socialprofile.models.IntegerRangeField(
                blank=True, default=1, null=True, verbose_name="Sort Order"
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="title",
            field=models.CharField(
                blank=True,
                default="Member",
                max_length=500,
                null=True,
                verbose_name="Title",
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="url",
            field=models.URLField(
                blank=True,
                help_text="Where can we find out more about you?",
                max_length=500,
                null=True,
                verbose_name="Homepage",
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="username",
            field=models.CharField(
                error_messages={"unique": "A user with that username already exists."},
                help_text="Required. 30 characters or fewer. Letters, digits and @/./+/-/_ only.",
                max_length=30,
                unique=True,
                validators=[
                    django.core.validators.RegexValidator(
                        "^[\\w.@+-]+$",
                        "Enter a valid username. This value may contain only letters, numbers and @/./+/-/_ characters.",
                    )
                ],
                verbose_name="Username",
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="visible",
            field=models.NullBooleanField(
                default=False,
                help_text="Designates whether this user should be visible by all. Unselect this make your account private!",
                verbose_name="Visible in the Public Pages",
            ),
        ),
    ]



# Generated by Django 1.11.4 on 2017-08-10 18:43

import user_sessions.models
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("axes", "0003_auto_20160322_0929"),
        ("oauth2_provider", "0002_auto_20190406_1805"),
        ("social_django", "0006_partial"),
        ("otp_totp", "0001_initial"),
        ("user_sessions", "0003_auto_20161205_1516"),
        ("two_factor", "0005_auto_20160224_0450"),
        ("otp_static", "0001_initial"),
        ("socialprofile", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="ProxyAccessAttempt",
            fields=[],
            options={
                "verbose_name": "Monitor: access attempt",
                "proxy": True,
                "verbose_name_plural": "Monitor: access attempts",
                "indexes": [],
            },
            bases=("axes.accessattempt",),
        ),
        migrations.CreateModel(
            name="ProxyAccessLog",
            fields=[],
            options={
                "verbose_name": "Monitor: access log",
                "proxy": True,
                "verbose_name_plural": "Monitor: access logs",
                "indexes": [],
            },
            bases=("axes.accesslog",),
        ),
        migrations.CreateModel(
            name="ProxyAccessToken",
            fields=[],
            options={
                "verbose_name": "Token: access token",
                "proxy": True,
                "verbose_name_plural": "Token: access tokens",
                "indexes": [],
            },
            bases=("oauth2_provider.accesstoken",),
        ),
        migrations.CreateModel(
            name="ProxyApplication",
            fields=[],
            options={
                "verbose_name": "Token: application",
                "proxy": True,
                "verbose_name_plural": "Token: applications",
                "indexes": [],
            },
            bases=("oauth2_provider.application",),
        ),
        migrations.CreateModel(
            name="ProxyAssociation",
            fields=[],
            options={
                "verbose_name": "OAuth: association",
                "proxy": True,
                "verbose_name_plural": "OAuth: associations",
                "indexes": [],
            },
            bases=("social_django.association",),
        ),
        migrations.CreateModel(
            name="ProxyGrant",
            fields=[],
            options={
                "verbose_name": "Token: grant",
                "proxy": True,
                "verbose_name_plural": "Token: grants",
                "indexes": [],
            },
            bases=("oauth2_provider.grant",),
        ),
        migrations.CreateModel(
            name="ProxyNonce",
            fields=[],
            options={
                "verbose_name": "OAuth: nonce",
                "proxy": True,
                "verbose_name_plural": "OAuth: nonces",
                "indexes": [],
            },
            bases=("social_django.nonce",),
        ),
        # migrations.CreateModel(
        #     name="ProxyPhoneDevice",
        #     fields=[],
        #     options={
        #         "verbose_name": "Otp: phone device",
        #         "proxy": True,
        #         "verbose_name_plural": "Otp: phone devices",
        #         "indexes": [],
        #     },
        #     bases=("two_factor.phonedevice",),
        # ),
        migrations.CreateModel(
            name="ProxyRefreshToken",
            fields=[],
            options={
                "verbose_name": "Token: refresh token",
                "proxy": True,
                "verbose_name_plural": "Token: refresh tokens",
                "indexes": [],
            },
            bases=("oauth2_provider.refreshtoken",),
        ),
        migrations.CreateModel(
            name="ProxySession",
            fields=[],
            options={
                "verbose_name": "Monitor: sessione",
                "proxy": True,
                "verbose_name_plural": "Monitor: sessioni",
                "indexes": [],
            },
            bases=("user_sessions.session",),
            managers=[
                ("objects", user_sessions.models.SessionManager()),
            ],
        ),
        migrations.CreateModel(
            name="ProxyStaticDevice",
            fields=[],
            options={
                "verbose_name": "Otp: static device",
                "proxy": True,
                "verbose_name_plural": "Otp: static devices",
                "indexes": [],
            },
            bases=("otp_static.staticdevice",),
        ),
        migrations.CreateModel(
            name="ProxyTOTPDevice",
            fields=[],
            options={
                "verbose_name": "Otp: TOTP device",
                "proxy": True,
                "verbose_name_plural": "Otp: TOTP devices",
                "indexes": [],
            },
            bases=("otp_totp.totpdevice",),
        ),
        migrations.CreateModel(
            name="ProxyUserSocialAuth",
            fields=[],
            options={
                "verbose_name": "OAuth: user social auth",
                "proxy": True,
                "verbose_name_plural": "OAuth: user social auths",
                "indexes": [],
            },
            bases=("social_django.usersocialauth",),
        ),
    ]



import django.contrib.auth.models
import django.core.validators
import image_cropping.fields
from django.db import migrations, models

import socialprofile.models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("auth", "0011_update_proxy_permissions"),
    ]

    operations = [
        migrations.CreateModel(
            name="SocialProfile",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("password", models.CharField(max_length=128, verbose_name="password")),
                (
                    "last_login",
                    models.DateTimeField(
                        blank=True, null=True, verbose_name="last login"
                    ),
                ),
                (
                    "is_superuser",
                    models.BooleanField(
                        default=False,
                        help_text="Designates that this user has all permissions without explicitly assigning them.",
                        verbose_name="superuser status",
                    ),
                ),
                (
                    "username",
                    models.CharField(
                        error_messages={
                            b"unique": "A user with that username already exists."
                        },
                        help_text="Required. 30 characters or fewer. Letters, digits and @/./+/-/_ only.",
                        max_length=30,
                        unique=True,
                        validators=[
                            django.core.validators.RegexValidator(
                                b"^[\\w.@+-]+$",
                                "Enter a valid username. This value may contain only letters, numbers and @/./+/-/_ characters.",
                            )
                        ],
                        verbose_name="Username",
                    ),
                ),
                (
                    "first_name",
                    models.CharField(
                        blank=True, max_length=30, verbose_name="First Name"
                    ),
                ),
                (
                    "last_name",
                    models.CharField(
                        blank=True, max_length=30, verbose_name="Last Name"
                    ),
                ),
                (
                    "email",
                    models.EmailField(
                        max_length=254, unique=True, verbose_name="Email Address"
                    ),
                ),
                (
                    "is_staff",
                    models.BooleanField(
                        default=False,
                        help_text="Designates whether the user can log into this admin site.",
                        verbose_name="staff status",
                    ),
                ),
                (
                    "is_active",
                    models.BooleanField(
                        default=True,
                        help_text="Designates whether this user should be treated as active. Unselect this instead of deleting accounts.",
                        verbose_name="active",
                    ),
                ),
                (
                    "date_of_birth",
                    models.DateField(null=True, verbose_name="Date of Birth"),
                ),
                (
                    "date_joined",
                    models.DateTimeField(auto_now_add=True, verbose_name="Date Joined"),
                ),
                (
                    "date_modified",
                    models.DateTimeField(auto_now=True, verbose_name="Date Updated"),
                ),
                (
                    "country",
                    models.CharField(
                        blank=True, max_length=255, verbose_name="Country"
                    ),
                ),
                (
                    "gender",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("male", "Male"),
                            ("female", "Female"),
                            ("other", "Other"),
                            (b"", b""),
                        ],
                        max_length=10,
                        verbose_name="Gender",
                    ),
                ),
                (
                    "url",
                    models.URLField(
                        blank=True,
                        help_text="Where can we find out more about you?",
                        max_length=500,
                        verbose_name="Homepage",
                    ),
                ),
                (
                    "image_url",
                    models.URLField(
                        blank=True, max_length=500, verbose_name="Avatar Picture"
                    ),
                ),
                (
                    "image",
                    image_cropping.fields.ImageCropField(
                        blank=True, upload_to=b"user/"
                    ),
                ),
                (
                    "cropping",
                    image_cropping.fields.ImageRatioField(
                        "image",
                        "120x100",
                        adapt_rotation=False,
                        allow_fullsize=True,
                        free_crop=False,
                        help_text=None,
                        hide_image_field=False,
                        size_warning=False,
                        verbose_name="cropping",
                    ),
                ),
                (
                    "cropping_free",
                    image_cropping.fields.ImageRatioField(
                        "image",
                        "300x300",
                        adapt_rotation=False,
                        allow_fullsize=False,
                        free_crop=True,
                        help_text=None,
                        hide_image_field=False,
                        size_warning=True,
                        verbose_name="cropping free",
                    ),
                ),
                (
                    "description",
                    models.TextField(
                        blank=True,
                        help_text="Tell us about yourself!",
                        verbose_name="Description",
                    ),
                ),
                (
                    "manually_edited",
                    models.BooleanField(default=False, verbose_name="Manually Edited"),
                ),
                (
                    "phone_number",
                    models.CharField(
                        blank=True,
                        max_length=16,
                        validators=[
                            django.core.validators.RegexValidator(
                                message="Phone number must be entered in the format: '+999999999'. Up to 15 digits allowed.",
                                regex=b"^\\+\\d{8,15}$",
                            )
                        ],
                    ),
                ),
                (
                    "sort",
                    socialprofile.models.IntegerRangeField(
                        blank=True, default=1, verbose_name="Sort Order"
                    ),
                ),
                (
                    "visible",
                    models.BooleanField(
                        default=False, verbose_name="Visible in the Public Pages"
                    ),
                ),
                (
                    "title",
                    models.CharField(blank=True, max_length=500, verbose_name="Title"),
                ),
                (
                    "role",
                    models.CharField(blank=True, max_length=500, verbose_name="Role"),
                ),
                ("function_01", models.CharField(blank=True, max_length=200)),
                ("function_02", models.CharField(blank=True, max_length=200)),
                ("function_03", models.CharField(blank=True, max_length=200)),
                ("function_04", models.CharField(blank=True, max_length=200)),
                ("function_05", models.CharField(blank=True, max_length=200)),
                ("function_06", models.CharField(blank=True, max_length=200)),
                ("function_07", models.CharField(blank=True, max_length=200)),
                ("function_08", models.CharField(blank=True, max_length=200)),
                ("function_09", models.CharField(blank=True, max_length=200)),
                ("function_10", models.CharField(blank=True, max_length=200)),
                (
                    "editByGoogle",
                    models.BooleanField(
                        default=False, verbose_name="Google Edit User's Info"
                    ),
                ),
                (
                    "google_isPlusUser",
                    models.BooleanField(default=False, verbose_name="Google Plus"),
                ),
                (
                    "google_plusUrl",
                    models.URLField(
                        blank=True, max_length=500, verbose_name="Google Plus Page"
                    ),
                ),
                (
                    "google_circledByCount",
                    models.IntegerField(
                        blank=True, default=0, verbose_name="Google Circle byCount"
                    ),
                ),
                (
                    "google_language",
                    models.CharField(
                        blank=True, max_length=2, verbose_name="Google Language"
                    ),
                ),
                (
                    "google_kind",
                    models.CharField(
                        blank=True, max_length=2, verbose_name="Google Kind"
                    ),
                ),
                (
                    "google_verified",
                    models.BooleanField(default=False, verbose_name="Google Verified"),
                ),
            ],
            options={
                "ordering": ["username"],
                "abstract": False,
                "verbose_name_plural": "Social Profiles",
                "proxy": False,
                "verbose_name": "Social Profile",
                "swappable": "AUTH_USER_MODEL",
            },
            managers=[
                ("objects", socialprofile.models.SocialProfileManager()),
            ],
        ),
        migrations.CreateModel(
            name="Group",
            fields=[],
            options={
                "proxy": True,
                "indexes": [],
            },
            bases=("auth.group",),
            managers=[
                ("objects", django.contrib.auth.models.GroupManager()),
            ],
        ),
        migrations.AddField(
            model_name="socialprofile",
            name="groups",
            field=models.ManyToManyField(
                blank=True,
                help_text="The groups this user belongs to. A user will get all permissions granted to each of their groups.",
                related_name="user_set",
                related_query_name="user",
                to="auth.Group",
                verbose_name="groups",
            ),
        ),
        migrations.AddField(
            model_name="socialprofile",
            name="user_permissions",
            field=models.ManyToManyField(
                blank=True,
                help_text="Specific permissions for this user.",
                related_name="user_set",
                related_query_name="user",
                to="auth.Permission",
                verbose_name="user permissions",
            ),
        ),
    ]



"""Unit Tests for the socialprofile module"""

# pylint: disable=R0904, C0103

import logging

from django.test import TestCase

from socialprofile.models import SocialProfile

LOGGER = logging.getLogger(name="socialprofile")


class SocialProfileTestCase(TestCase):
    """Test Case for Social Profile"""

    def setUp(self):
        """Set up common assets for tests"""
        LOGGER.debug("SocialProfile Tests setUp")
        self.user1 = SocialProfile.objects.create_user(
            "user1", "user1@user1.com", "user1password"
        )
        self.user1.gender = "unknown"
        self.user1.url = "http://test.com"
        self.user1.description = "Test User 1"
        self.user1.image_url = (
            "http://www.gravatar.com/avatar/00000000000000000000000000000000?d=mm"
        )
        self.user1.save()
        # self.sp1 = SocialProfile.objects.update(id=self.user1,
        # gender="Male",
        # url="http://test.com",
        # description="Test User 1",
        # image_url="http://www.gravatar.com/avatar/00000000000000000000000000000000?d=mm"
        # )
        # self.sa1 = UserSocialAuth.objects.create(user=self.user1, provider='google-oauth2', uid='user1@user1.com')
        # self.sa2 = UserSocialAuth.objects.create(user=self.user1, provider='facebook', uid='user1@user1.com')
        # self.sa2 = UserSocialAuth.objects.create(user=self.user1, provider='twitter', uid='user1@user1.com')

    def tearDown(self):
        """Remove Test Data"""
        LOGGER.debug("SocialProfile Tests tearDown")
        self.user1.delete()

    # def test_redirect_urls(self):
    # """Test that redirects kicking in when trying to go to secure page."""
    # LOGGER.debug("SocialProfile Test Redirect URLs")
    # response = self.client.get('/secure/', follow=True)
    # self.assertRedirects(response, "http://testserver/sp/select/?next=/secure/")
    #
    # def test_view_profile(self):
    # """Test to see if profile for user1 can be viewed anon and logged in"""
    # LOGGER.debug("Test GET /sp/view/user1/ for anon user")
    # anon_view_response = self.client.get('/sp/view/user1/')
    # self.assertContains(anon_view_response, "Test User 1")
    #
    # LOGGER.debug("Test GET /sp/ for anon user")
    # anon_view_generic_response = self.client.get('/sp/')
    # self.assertEqual(404, anon_view_generic_response.status_code)
    #
    # LOGGER.debug("Test GET /sp/view/ for anon user")
    # anon_view_generic_response_2 = self.client.get('/sp/view/')
    # self.assertEqual(404, anon_view_generic_response_2.status_code)
    #
    # LOGGER.debug("Test GET /sp/view/user1/ for logged in user")
    # self.client.login(username='user1', password='user1password')
    # logged_in_view_response = self.client.get('/sp/view/user1/')
    # self.assertContains(logged_in_view_response, "Test User 1")
    #
    # LOGGER.debug("Test GET /sp/ for logged in user")
    # logged_in_view_generic_response = self.client.get('/sp/')
    # self.assertContains(logged_in_view_generic_response, "Test User 1")
    #
    # LOGGER.debug("Test GET /sp/view/ for logged in user")
    # logged_in_view_generic_response_2 = self.client.get('/sp/view/')
    # self.assertEqual(404, logged_in_view_generic_response_2.status_code)
    #
    # LOGGER.debug("Test GET /sp/view/ for logged in user")
    # logged_in_view_generic_response_2 = self.client.get('/sp/view/')
    # self.assertEqual(404, logged_in_view_generic_response_2.status_code)
    #
    # LOGGER.debug("Test POST to /sp/view/ for logged in user")
    # logged_in_view_post_response = self.client.post('/sp/', {'user': 1}, follow=True)
    # self.assertEqual(405, logged_in_view_post_response.status_code)  # HTTP POST Not Allowed
    #
    # def test_socialprofile_permalink(self):
    # """Test the permalink method of SocialProfile"""
    # LOGGER.debug("Test SocialProfile Permalink")
    # profile = SocialProfile.objects.get(id=self.user1.id)
    # permalink = profile.get_absolute_url()
    # self.assertEqual("/sp/view/user1/", permalink)
    # anon_view_response = self.client.get(permalink)
    # self.assertContains(anon_view_response, "Test User 1")
    #
    # def test_edit_profile(self):
    # """Test to see if profile for user1 can be edited anon and logged in"""
    # LOGGER.debug("Test GET /sp/edit/user1/ for anon user")
    # anon_edit_response = self.client.get('/sp/edit/user1/')
    # self.assertEqual(404, anon_edit_response.status_code)
    #
    # LOGGER.debug("Test GET /sp/edit/ for anon user")
    # anon_edit_response_2 = self.client.get('/sp/edit/')
    # self.assertRedirects(anon_edit_response_2, "http://testserver/sp/select/?next=/sp/edit/")
    #
    # LOGGER.debug("Test POST /sp/edit/ for anon user")
    # anon_edit_response_2 = self.client.post('/sp/edit/', {'user': 1}, follow=True)
    # self.assertRedirects(anon_edit_response_2, "http://testserver/sp/select/?next=/sp/edit/")
    #
    # LOGGER.debug("Test GET /sp/edit/user1/ for logged in user")
    # self.client.login(username='user1', password='user1password')
    # anon_edit_response = self.client.get('/sp/edit/user1/')
    # self.assertEqual(404, anon_edit_response.status_code)
    #
    # LOGGER.debug("Test GET /sp/edit/ for logged in user")
    # logged_in_edit_response = self.client.get('/sp/edit/')
    # self.assertContains(logged_in_edit_response, "Test User 1")
    #
    # LOGGER.debug("Test POST /sp/edit/ for logged in user")
    # post_data = {
    # 'user': 1,
    # 'username': 'user2',
    # 'email': 'user1@test.com',
    # 'first_name': 'Test',
    # 'last_name': 'User',
    # 'description': "Test1 User",
    # 'image_url': 'http://foo.com',
    # 'url': 'http://user1.com',
    # 'returnTo': '/secure/'
    # }
    # logged_in_edit_response_2 = self.client.post('/sp/edit/', post_data, follow=True)
    # self.assertContains(logged_in_edit_response_2, "updated")
    # user = SocialProfile.objects.get(username='user2')
    # self.assertEqual('user2', str(user.social_profile))
    #
    # LOGGER.debug("Test Invalid Form Error")
    # post_data = {
    # 'username': 'user2',
    # 'email': 'user1@test.com',
    # 'gender': 'Robot',
    # 'first_name': 'Test',
    # 'last_name': 'User',
    # 'description': "Test1 User",
    # 'image_url': 'http://foo.com',
    # 'url': 'http://user1.com',
    # 'returnTo': '/secure/'
    # }
    # logged_in_edit_response_3 = self.client.post('/sp/edit/', post_data, follow=True)
    # self.assertContains(logged_in_edit_response_3, "NOT")
    #
    # def test_delete_user(self):
    # """Test the views that enable deleting users/socialprofiles"""
    # LOGGER.debug("Test GET /sp/delete/ for logged in user")
    # self.client.login(username='user1', password='user1password')
    # logged_in_delete_response = self.client.get('/sp/delete/')
    # self.assertContains(logged_in_delete_response, "Really Delete")
    #
    # LOGGER.debug("Test POST to /sp/delete/ for logged in user")
    # logged_in_delete_post_response = self.client.post('/sp/delete/', {'user': 1}, follow=True)
    # self.assertRedirects(logged_in_delete_post_response, '/')
    #
    # LOGGER.debug("Test GET /sp/view/user1/ for deleted user")
    # deleted_response = self.client.get('/sp/view/user1/')
    # LOGGER.debug(deleted_response)
    # self.assertEqual(404, deleted_response.status_code)



# Generated by Django 2.2.19 on 2021-10-30 22:42

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("socialprofile", "0005_auto_20211030_2237"),
    ]

    operations = [
        migrations.AlterField(
            model_name="socialprofile",
            name="google_language",
            field=models.CharField(
                blank=True, max_length=10, null=True, verbose_name="Google Language"
            ),
        ),
    ]



# Generated by Django 2.2.24 on 2021-11-29 15:13

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("socialprofile", "0011_delete_group"),
    ]

    operations = [
        migrations.AlterField(
            model_name="socialprofile",
            name="phone_number",
            field=models.CharField(
                blank=True,
                max_length=16,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        message="Phone number must be entered in the format: '+999999999'. Up to 15 digits allowed.",
                        regex="^\\+\\d{8,15}$",
                    )
                ],
                verbose_name="Phone Number",
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="visible",
            field=models.BooleanField(
                choices=[(True, "Yes"), (False, "No")],
                default=True,
                help_text="Designates whether this user should be visible by all. Unselect this make your account private!",
                verbose_name="Visible in the Public Pages",
            ),
        ),
    ]



# Generated by Django 2.2.24 on 2021-11-29 22:05

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("socialprofile", "0016_auto_20211129_2259"),
    ]

    operations = [
        migrations.RenameField(
            model_name="socialprofile",
            old_name="username_facebook",
            new_name="facebook_username",
        ),
        migrations.RenameField(
            model_name="socialprofile",
            old_name="username_instagram",
            new_name="instagram_username",
        ),
    ]



SITE_ID = 1

# Social Profile
### Custom Social Profile User
AUTH_USER_MODEL = "socialprofile.SocialProfile"
SOCIAL_AUTH_GOOGLE_OAUTH_SCOPE = [
    "https://www.googleapis.com/auth/drive",
    "https://www.googleapis.com/auth/userinfo.profile",
]

SOCIAL_AUTH_PIPELINE = (
    "social.pipeline.social_auth.social_details",
    "social.pipeline.social_auth.social_uid",
    "social.pipeline.social_auth.auth_allowed",
    "social.pipeline.social_auth.social_user",
    "social.pipeline.user.get_username",
    "social.pipeline.mail.mail_validation",
    "social.pipeline.user.create_user",
    "social.pipeline.social_auth.associate_user",
    "social.pipeline.debug.debug",
    "social.pipeline.social_auth.load_extra_data",
    "social.pipeline.user.user_details",
    "socialprofile.pipeline.socialprofile_extra_values",
)

SOCIAL_AUTH_LOGIN_REDIRECT_URL = "/"
SOCIAL_AUTH_NEW_USER_REDIRECT_URL = "/"
SOCIAL_AUTH_NEW_ASSOCIATION_REDIRECT_URL = "/select/"
DEFAULT_RETURNTO_PATH = "/select/"

# Core Authentication Settings
LOGIN_URL = "/select/"
LOGIN_REDIRECT_URL = "/"
LOGIN_ERROR_URL = "/select/"

SESSION_SERIALIZER = "django.contrib.sessions.serializers.PickleSerializer"

AUTHENTICATION_BACKENDS = (
    # 'django.contrib.auth.backends.ModelBackend',  # Comment if you want DISABLE Login Form (Password)
    "social.backends.amazon.AmazonOAuth2",
    "social.backends.angel.AngelOAuth2",
    "social.backends.aol.AOLOpenId",
    "social.backends.appsfuel.AppsfuelOAuth2",
    "social.backends.beats.BeatsOAuth2",
    "social.backends.behance.BehanceOAuth2",
    "social.backends.belgiumeid.BelgiumEIDOpenId",
    "social.backends.bitbucket.BitbucketOAuth",
    "social.backends.box.BoxOAuth2",
    "social.backends.clef.ClefOAuth2",
    "social.backends.coinbase.CoinbaseOAuth2",
    # 'social.backends.coursera.CourseraOAuth2',
    "social.backends.dailymotion.DailymotionOAuth2",
    "social.backends.disqus.DisqusOAuth2",
    "social.backends.douban.DoubanOAuth2",
    # 'social.backends.dropbox.DropboxOAuth',
    "social.backends.dropbox.DropboxOAuth2",
    # 'social.backends.eveonline.EVEOnlineOAuth2',
    "social.backends.evernote.EvernoteSandboxOAuth",
    "social.backends.facebook.FacebookAppOAuth2",
    "social.backends.facebook.FacebookOAuth2",
    "social.backends.fedora.FedoraOpenId",
    "social.backends.fitbit.FitbitOAuth",
    "social.backends.flickr.FlickrOAuth",
    "social.backends.foursquare.FoursquareOAuth2",
    "social.backends.github.GithubOAuth2",
    # 'social.backends.google.GoogleOAuth',
    "social.backends.google.GoogleOAuth2",
    # 'social.backends.google.GoogleOpenId',
    # 'social.backends.google.GooglePlusAuth',
    # 'social.backends.google.GoogleOpenIdConnect',
    "social.backends.instagram.InstagramOAuth2",
    "social.backends.jawbone.JawboneOAuth2",
    "social.backends.kakao.KakaoOAuth2",
    "social.backends.linkedin.LinkedinOAuth",
    "social.backends.linkedin.LinkedinOAuth2",
    "social.backends.live.LiveOAuth2",
    "social.backends.livejournal.LiveJournalOpenId",
    "social.backends.mailru.MailruOAuth2",
    "social.backends.mendeley.MendeleyOAuth",
    "social.backends.mendeley.MendeleyOAuth2",
    # 'social.backends.mineid.MineIDOAuth2',
    "social.backends.mixcloud.MixcloudOAuth2",
    # 'social.backends.nationbuilder.NationBuilderOAuth2',
    "social.backends.odnoklassniki.OdnoklassnikiOAuth2",
    "social.backends.open_id.OpenIdAuth",
    "social.backends.openstreetmap.OpenStreetMapOAuth",
    "social.backends.persona.PersonaAuth",
    "social.backends.podio.PodioOAuth2",
    "social.backends.rdio.RdioOAuth1",
    "social.backends.rdio.RdioOAuth2",
    "social.backends.readability.ReadabilityOAuth",
    "social.backends.reddit.RedditOAuth2",
    "social.backends.runkeeper.RunKeeperOAuth2",
    "social.backends.skyrock.SkyrockOAuth",
    "social.backends.soundcloud.SoundcloudOAuth2",
    "social.backends.spotify.SpotifyOAuth2",
    "social.backends.stackoverflow.StackoverflowOAuth2",
    "social.backends.steam.SteamOpenId",
    "social.backends.stocktwits.StocktwitsOAuth2",
    "social.backends.stripe.StripeOAuth2",
    "social.backends.suse.OpenSUSEOpenId",
    "social.backends.thisismyjam.ThisIsMyJamOAuth1",
    "social.backends.trello.TrelloOAuth",
    "social.backends.tripit.TripItOAuth",
    "social.backends.tumblr.TumblrOAuth",
    "social.backends.twilio.TwilioAuth",
    "social.backends.twitter.TwitterOAuth",
    "social.backends.vk.VKOAuth2",
    "social.backends.weibo.WeiboOAuth2",
    # 'social.backends.wunderlist.WunderlistOAuth2',
    "social.backends.xing.XingOAuth",
    "social.backends.yahoo.YahooOAuth",
    "social.backends.yahoo.YahooOpenId",
    "social.backends.yammer.YammerOAuth2",
    "social.backends.yandex.YandexOAuth2",
    "social.backends.vimeo.VimeoOAuth1",
    "social.backends.lastfm.LastFmAuth",
    "social.backends.moves.MovesOAuth2",
    # 'social.backends.vend.VendOAuth2',
    "social.backends.email.EmailAuth",
    "social.backends.username.UsernameAuth",
)

SECRET_KEY = "fake-key"
DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": "tests.db",
        "SUPPORTS_TRANSACTIONS": "false",
    }
}
ROOT_URLCONF = "socialprofile.urls"
INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.sites",
    "django.contrib.staticfiles",
    "django_errors",
    "social.apps.django_app.default",
    "django_countries",
    "socialprofile",
]



# Generated by Django 2.2.24 on 2021-11-21 14:51

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("socialprofile", "0010_auto_20211101_1214"),
    ]

    operations = [
        migrations.DeleteModel(
            name="Group",
        ),
    ]



# Generated by Django 2.2.11 on 2020-04-03 13:25

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("socialprofile", "0006_auto_20211030_2242"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="proxysession",
            options={
                "verbose_name": "Monitor: session",
                "verbose_name_plural": "Monitor: sessions",
            },
        ),
    ]



"""Django forms for the socialprofile application"""

import logging

from crispy_forms.bootstrap import (
    Accordion,
    AccordionGroup,
    Field,
    InlineRadios,
    PrependedText,
    UneditableField,
)
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Column, Layout, Row, Submit
from django import forms
from django.conf import settings
from django.utils.html import strip_tags
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from django_countries.widgets import CountrySelectWidget

from .models import SocialProfile

LOGGER = logging.getLogger(name="socialprofile.forms")


class CustomImageField(Field):
    template = "socialprofile/image_thumbnail.html"


class SocialProfileForm(forms.ModelForm):
    """Master form for editing the user's profile"""

    # user = forms.IntegerField(widget=forms.HiddenInput, required=True)
    returnTo = forms.CharField(
        widget=forms.HiddenInput, required=False, initial="/"
    )  # URI to Return to after save
    edited_by_user = forms.BooleanField(
        widget=forms.HiddenInput, required=False, initial=True
    )

    date_of_birth = forms.DateField(
        label=_("Date of Birth"),
        widget=forms.TextInput(attrs={"type": "date"}),
        required=False,
    )

    class Meta:
        """Configuration for the ModelForm"""

        model = SocialProfile
        # exclude = [
        #     "is_staff",
        #     "is_superuser",
        #     "is_active",
        #     "password",
        #     "edited_by_provider",
        # ]
        fields = [
            "username",
            "first_name",
            "last_name",
            "gender",
            "date_of_birth",
            "email",
            "phone_number",
            "url",
            "image_avatar",
            "image",
            "cropping",
            "image_avatar_predef",
            "google_username",
            "twitter_username",
            "instagram_username",
            "facebook_username",
            "google_email",
            "twitter_email",
            "instagram_email",
            "facebook_email",
            "live_email",
            "country",
            "city",
            "postalcode",
            "address",
            "description",
            "visible",
        ]
        # todo: Avoid bound_field = form[field] missing fields
        # if "google" in str(settings.AUTHENTICATION_BACKENDS):
        #     fields.append("google_email")
        # if "twitter" in str(settings.AUTHENTICATION_BACKENDS):
        #     fields.append("twitter_email")
        # if "instagram" in str(settings.AUTHENTICATION_BACKENDS):
        #     fields.append("instagram_email")
        # if "facebook" in str(settings.AUTHENTICATION_BACKENDS):
        #     fields.append("facebook_email")
        # if "live" in str(settings.AUTHENTICATION_BACKENDS):
        #     fields.append("live_email")

        widgets = {
            "country": CountrySelectWidget(layout="{widget}"),
            "gender": forms.RadioSelect,
            "visible": forms.RadioSelect,
            # "image": ImageCropWidget,
        }

    # Don't let through for security reasons, user should be based on logged in user only
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.form_id = "sp-edit-form"
        self.helper.render_hidden_fields = True
        self.helper.include_media = False
        # self.helper.form_show_errors = True
        # self.helper.use_custom_control = True

        # disable all mail's fields to avoid hijacking inside html
        for nam, field in self.fields.items():
            if "email" in nam:
                field.disabled = True

        # self.helper.form_show_labels = False
        self.fields["username"].label = False
        # self.fields["country"].label = False
        self.fields["twitter_username"].label = False
        self.fields["facebook_username"].label = False
        self.fields["instagram_username"].label = False

        self.fields["image_avatar"].label = False
        self.fields["image"].label = False
        self.fields["image_avatar_predef"].label = False

        self.helper.label_class = "labels"
        self.helper.field_class = "col-md-12"
        # self.helper.form_class = 'form-vertical'
        # # self.helper.label_class = "col-md-2"
        # # self.helper.field_class = "col-md-10"

        self.helper.layout = Layout(
            Row(
                Field("username"),
                css_class="row mt-2",
            ),
            Row(
                Column("first_name", css_class="col-md-6"),
                Column("last_name", css_class="col-md-6"),
                css_class="row mt-2",
            ),
            Row(
                InlineRadios("gender", css_class="col-md-6"),
                Field("date_of_birth"),
                css_class="row mt-3",
            ),
            Accordion(
                AccordionGroup(
                    _("Contact"),
                    # UneditableField("email"),
                    Field("phone_number"),
                    Field("url"),
                    active=False,
                ),
                # AccordionGroup(
                #     _("Avatar"),
                #     InlineRadios("image_avatar", css_class="col-md-6"),
                #     TabHolder(
                #         Tab(
                #             _("Custom"),
                #             # CustomImageField('image'),
                #             Field('image'),
                #             Field("cropping"),
                #             css_class="row mt-3",
                #         ),
                #         Tab(
                #             _("Pre-Defined"),
                #             Field("image_avatar_predef"),
                #             css_class="row mt-3",
                #         ),
                #     ),
                #
                # ),
                AccordionGroup(
                    _("Socials"),
                    Row(
                        PrependedText(
                            "twitter_username",
                            "@",
                            placeholder=f"Twitter {_('Username')}",
                        ),
                        PrependedText(
                            "instagram_username",
                            "@",
                            placeholder=f"Instagram {_('Username')}",
                        ),
                        PrependedText(
                            "facebook_username",
                            "@",
                            placeholder=f"Facebook {_('Username')}",
                        ),
                        css_class="row mt-3",
                    ),
                ),
                AccordionGroup(
                    _("Mails"),
                    Row(
                        Field("email", placeholder=f"Mail", readonly=True),
                        Field(
                            "google_email", placeholder=f"Google Mail", readonly=True
                        ),
                        Field(
                            "twitter_email", placeholder=f"Twitter Mail", readonly=True
                        ),
                        Field(
                            "instagram_email",
                            placeholder=f"Instagram Mail",
                            readonly=True,
                        ),
                        Field(
                            "facebook_email",
                            placeholder=f"Facebook Mail",
                            readonly=True,
                        ),
                        css_class="row mt-3",
                    ),
                ),
                AccordionGroup(
                    _("Address"),
                    PrependedText(
                        "country",
                        mark_safe(
                            f'<img class="country-select-flag" id="flag_id_country" '
                            f'src="{settings.STATIC_URL}flags/it.gif" '
                            f'style="margin: 4px 4px 0" width="25" height="20">'
                        ),
                    ),
                    Row(
                        Column("city", css_class="col-md-6"),
                        Column("postalcode", css_class="col-md-6"),
                        css_class="row mt-2",
                    ),
                    Field("address"),
                ),
                AccordionGroup(
                    _("Info"),
                    Row(
                        Field("description"),
                        css_class="row mt-3",
                    ),
                ),
                AccordionGroup(
                    _("Security"),
                    InlineRadios("visible"),
                ),
            ),
            # ButtonHolder(
            Row(
                Submit(
                    "submit",
                    _("Save"),
                    css_class="text-center profile-button",
                ),
                css_class="row mt-5",
            ),
        )
        # self.helper.fields['email'].initial = "test"

        # for nam, field in self.fields.items():
        #     field.disabled = True

    def clean_description(self):
        """Automatically called by Django, this method 'cleans' the description, e.g. stripping HTML out of desc"""

        LOGGER.debug("socialprofile.forms.SocialProfileForm.clean_description")

        return strip_tags(self.cleaned_data["description"])

    def clean(self):
        """Automatically called by Django, this method 'cleans' the whole form"""

        LOGGER.debug("socialprofile.forms.SocialProfileForm.clean")

        if self.changed_data:
            self.cleaned_data["edited_by_user"] = True



# Generated by Django 2.2.24 on 2021-12-03 08:58

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("socialprofile", "0019_auto_20211202_2216"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="socialprofile",
            name="last_accessed",
        ),
    ]



"""Django App configuration for socialprofile"""

from django.apps import AppConfig


class SocialProfileConfig(AppConfig):
    """Default configuration for socialprofile"""

    default_auto_field = "django.db.models.AutoField"
    name = "socialprofile"
    verbose_name = "Social Profile"



# Generated by Django 2.2.24 on 2021-11-29 20:17

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("socialprofile", "0014_auto_20211129_1729"),
    ]

    operations = [
        migrations.RenameField(
            model_name="socialprofile",
            old_name="google_plusUrl",
            new_name="google_url",
        ),
        migrations.AddField(
            model_name="socialprofile",
            name="facebook_url",
            field=models.URLField(
                blank=True, max_length=500, null=True, verbose_name="Facebook Profile"
            ),
        ),
        migrations.AddField(
            model_name="socialprofile",
            name="twitter_url",
            field=models.URLField(
                blank=True, max_length=500, null=True, verbose_name="Twitter Profile"
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="edited_by_provider",
            field=models.NullBooleanField(
                default=False, verbose_name="Social provider edited user's info"
            ),
        ),
    ]



"""Django Views for the socialprofile module"""

import json
import logging

import sweetify
from django.conf import settings
from django.contrib import messages
from django.contrib.auth import REDIRECT_FIELD_NAME, get_user_model, login
from django.contrib.auth import logout as auth_logout
from django.contrib.auth.decorators import login_required
from django.core.exceptions import PermissionDenied
from django.http import Http404, HttpResponse, HttpResponseBadRequest
from django.shortcuts import get_object_or_404, redirect
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _

# from django.views.decorators.csrf import csrf_exempt
from django.views.generic import DeleteView, TemplateView, UpdateView
from oauth2_provider.contrib.rest_framework import TokenHasReadWriteScope
from rest_framework import permissions, viewsets
from social_core.backends.oauth import BaseOAuth1, BaseOAuth2
from social_core.backends.utils import load_backends
from social_django.utils import psa

from .decorators import render_to
from .forms import SocialProfileForm
from .models import SocialProfile

# from .serializers import SocialProfileSerializer, GroupSerializer
from .serializers import SocialProfileSerializer

LOGGER = logging.getLogger(name="socialprofile.views")

DEFAULT_RETURNTO_PATH = getattr(settings, "DEFAULT_RETURNTO_PATH", "/")


# todo: avoid alert with unique username with table column
# @csrf_exempt
# def check_username_exist(request):
#     username=request.POST.get("username")
#     user = get_user_model()
#     user_obj=user.objects.filter(username=username).exists()
#     if user_obj:
#         return HttpResponse(True)
#     else:
#         return HttpResponse(False)
#


# ViewSets define the view behavior.
class SocialProfileViewSet(viewsets.ModelViewSet):
    """Serialize SocialProfiles"""

    permission_classes = [permissions.IsAuthenticated, TokenHasReadWriteScope]
    queryset = SocialProfile.objects.all()
    serializer_class = SocialProfileSerializer


# class GroupViewSet(viewsets.ModelViewSet):
#     """Serialize Groups"""
#     permission_classes = [permissions.IsAuthenticated, TokenHasScope]
#     required_scopes = ['groups']
#     queryset = Group.objects.all()
#     serializer_class = GroupSerializer


def logout(request):
    """Logs out user"""
    auth_logout(request)
    return redirect("sp_select_page")


def context(**extra):
    return dict(
        {
            # "plus_id": getattr(settings, "SOCIAL_AUTH_GOOGLE_PLUS_KEY", None),
            # "plus_scope": " ".join(GooglePlusAuth.DEFAULT_SCOPE),
            "available_backends": load_backends(settings.AUTHENTICATION_BACKENDS),
        },
        **extra,
    )


@render_to("socialprofile/sp_account_select.html")
def home(request):
    """Home view, displays login mechanism"""
    if request.user.is_authenticated:
        return redirect("done")
    return context()


@login_required
@render_to("socialprofile/sp_account_select.html")
def done(request):
    """Login complete view, displays user data"""
    if request.user.is_authenticated:
        return context()
    return redirect("sp_select_page")


@psa("social:complete")
def ajax_auth(request, backend):
    if isinstance(request.backend, BaseOAuth1):
        token = {
            "oauth_token": request.REQUEST.get("access_token"),
            "oauth_token_secret": request.REQUEST.get("access_token_secret"),
        }
    elif isinstance(request.backend, BaseOAuth2):
        token = request.REQUEST.get("access_token")
    else:
        raise HttpResponseBadRequest(_("Wrong backend type"))
    user = request.backend.do_auth(token, ajax=True)
    login(request, user)
    data = {"id": user.id, "username": user.username}
    return HttpResponse(json.dumps(data), mimetype="application/json")


class SelectAuthView(TemplateView):
    """
    Lets users choose how they want to request access.

    url: /select
    """

    template_name = "socialprofile/sp_account_select.html"

    def get_context_data(self, **kwargs):
        """Ensure that 'next' gets passed along"""
        LOGGER.debug("socialprofile.views.SelectAuthView.get_context_data")

        next_url = self.request.GET.get(REDIRECT_FIELD_NAME, DEFAULT_RETURNTO_PATH)

        context = super().get_context_data(**kwargs)
        context["next_param"] = REDIRECT_FIELD_NAME
        context["next_url"] = next_url
        # context["plus_id"] = getattr(settings, "SOCIAL_AUTH_GOOGLE_PLUS_KEY", None)
        # context["plus_scope"] = " ".join(GooglePlusAuth.DEFAULT_SCOPE)
        context["available_backends"] = load_backends(settings.AUTHENTICATION_BACKENDS)
        return context


class SocialProfileWelcome(TemplateView):
    """
    New Profile Page

    url: /sp/new-profile
    """

    template_name = "socialprofile/sp_new_profile.html"


# class SocialProfileView(DetailView):
class SocialProfileView(TemplateView):
    """
    Profile View Page

    url: /sp/view
    """

    model = SocialProfile
    template_name = "socialprofile/sp_profile_view.html"
    http_method_names = {"get"}

    def get_context_data(self, **kwargs):
        """Load up the default data to show in the display form."""
        username = self.kwargs.get("username")
        if username:
            try:
                user = get_object_or_404(SocialProfile, username=username)
            except Exception as e:
                try:
                    user = get_object_or_404(SocialProfile, pk=username)
                except Exception as e:
                    user = get_object_or_404(SocialProfile, pk=self.request.user.pk)
        elif self.request.user.is_authenticated:
            user = get_object_or_404(SocialProfile, pk=self.request.user.pk)
        else:
            raise Http404  # Case where user gets to this view anonymously for non-existent user

        if self.request.user != user and user.visible:
            raise Http404  # Case where user set to be private

        return {
            "user": user,
            "available_backends": load_backends(settings.AUTHENTICATION_BACKENDS),
        }


class SocialProfileViewAll(TemplateView):
    """
    Profile View Page

    url: /sp/view/all
    """

    template_name = "socialprofile/sp_profile_view_all.html"
    http_method_names = {"get"}

    def dispatch(self, request, *args, **kwargs):
        if not getattr(
            settings,
            "SP_VIEW_PUBLIC",
            (
                getattr(settings, "SP_VIEW_PUBLIC_ONLY_ADMIN", False)
                and request.user.is_superuser
            ),
        ):
            return redirect("sp_profile_view")
        return super().dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        """Load up the default data to show in the display form."""
        return {
            "users": SocialProfile.objects.filter(visible=True),
        }


class SocialProfileEditView(UpdateView):
    """
    Profile Editing View

    url: /sp/edit
    """

    model = SocialProfile
    template_name = "socialprofile/sp_profile_edit.html"
    http_method_names = {"get", "post"}

    def get_context_data(self, **kwargs):
        """Load up the default data to show in the display form."""
        # user_form = UserForm(request.POST, instance=request.user)
        # username = self.kwargs.get("username")
        username = self.kwargs.get("pk")
        if username:
            try:
                user = get_object_or_404(SocialProfile, username=username)
            except Exception as e:
                try:
                    user = get_object_or_404(SocialProfile, pk=username)
                except Exception as e:
                    user = self.request.user
        elif self.request.user.is_authenticated:
            user = self.request.user
        else:
            raise Http404  # Case where user gets to this view anonymously for non-existent user

        if user != self.request.user:
            raise PermissionDenied()

        sp_form = SocialProfileForm(instance=user)
        return {
            "user": user,
            "available_backends": load_backends(settings.AUTHENTICATION_BACKENDS),
            "sp_form": sp_form,
        }

    def post(self, request, *args, **kwargs):
        # user_form = UserForm(request.POST, instance=request.user)
        return_to = self.request.POST.get("returnTo", DEFAULT_RETURNTO_PATH)

        custom_alerts = (
            getattr(settings, "SP_ALERT_LIBRARY", "sweetalert") == "sweetalert"
        )

        # username = self.kwargs.get("username")
        username = self.kwargs.get("pk")
        if username:
            try:
                user = get_object_or_404(SocialProfile, username=username)
            except Exception as e:
                try:
                    user = get_object_or_404(SocialProfile, pk=username)
                except Exception as e:
                    user = self.request.user

        if user != self.request.user:
            raise PermissionDenied()

        sp_form = SocialProfileForm(request.POST, instance=user)
        sp_form.initial["returnTo"] = return_to

        if sp_form.is_valid():
            try:
                sp_form.save()

                if custom_alerts:
                    sweetify.toast(
                        self.request,
                        _("Your profile has been updated."),
                        icon="success",
                        timer=3000,
                    )
                else:
                    messages.add_message(
                        self.request,
                        messages.SUCCESS,
                        _("Your profile has been updated."),
                    )
                return self.render_to_response(
                    {
                        "success": True,
                        "user": user,
                        "available_backends": load_backends(
                            settings.AUTHENTICATION_BACKENDS
                        ),
                        "sp_form": sp_form,
                    }
                )
            except Exception as e:
                if custom_alerts:
                    sweetify.toast(
                        self.request,
                        f"{_('ERROR: Your profile has NOT been updated!')} [{e}]",
                        icon="error",
                        timer=3000,
                    )
                else:
                    messages.add_message(
                        self.request,
                        messages.INFO,
                        f"{_('ERROR: Your profile has NOT been updated!')} [{e}]",
                    )

                return self.render_to_response(
                    {
                        "success": False,
                        "user": user,
                        "available_backends": load_backends(
                            settings.AUTHENTICATION_BACKENDS
                        ),
                        "sp_form": sp_form,
                    }
                )
        else:
            if custom_alerts:
                sweetify.toast(
                    self.request,
                    _("Your profile has NOT been updated!"),
                    icon="error",
                    timer=3000,
                )
                # multi = []
                # for x, err_msg in enumerate(sp_form.errors):
                #     multi.append({f"err_mess_{x}": dict(title='Error', icon='warning',
                #     text=err_msg, toast=True, timer=3000, timerProgressBar='true')})
                #     # sweetify.toast(
                #     #     self.request,
                #     #     err_msg,
                #     #     icon="warning",
                #     #     timer=3000,
                #     # )
                # if multi:
                #     sweetify.multiple(request, *multi[0])
            else:
                messages.add_message(
                    self.request, messages.INFO, _("Your profile has NOT been updated!")
                )

            return self.render_to_response(
                {
                    "success": False,
                    "user": user,
                    "available_backends": load_backends(
                        settings.AUTHENTICATION_BACKENDS
                    ),
                    "sp_form": sp_form,
                }
            )


class DeleteSocialProfileView(DeleteView):
    """
    Account Delete Confirm Modal View

    url: /delete
    """

    model = SocialProfile
    success_url = reverse_lazy("sp_logout_page")

    def get_object(self, queryset=None):
        """Get the object that we are going to delete"""
        return self.request.user



from django.conf import settings
from django.core.mail import mail_admins, send_mail
from django.db.models.signals import post_delete, post_save
from django.dispatch import receiver
from django.utils.html import strip_tags

from .models import SocialProfile


@receiver(post_delete, sender=SocialProfile)
def post_delete_user(sender, instance, *args, **kwargs):
    if instance.user:  # just in case user is not specified
        instance.user.delete()


@receiver(post_save, sender=SocialProfile)
def post_save_user(sender, instance, created, *args, **kwargs):
    # ToDo: notify new user
    if created:
        # notify the admin
        mail_admins(f"New user [{instance.user}]", "New user was created!")

        # ToDo: notify the user with a welcome custom message
        html_mail = f"""<!DOCTYPE html><html><body>
                        <h1> Welcome!</h1>
                    """
        html_mail += f"""</body></html>"""
        plain_message = strip_tags(html_mail)
        send_mail(
            f"Welcome {instance.user}!",
            plain_message,
            html_message=html_mail,
            from_email=settings.DEFAULT_FROM_EMAIL,
            recipient_list=instance.user.email,
        )



# Generated by Django 3.2.11 on 2022-01-10 23:08

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("socialprofile", "0023_auto_20211216_1745"),
    ]

    operations = [
        migrations.AlterField(
            model_name="socialprofile",
            name="edited_by_facebook",
            field=models.BooleanField(
                blank=True, default=False, verbose_name="Facebook edited"
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="edited_by_google",
            field=models.BooleanField(
                blank=True, default=False, verbose_name="Google edited"
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="edited_by_instagram",
            field=models.BooleanField(
                blank=True, default=False, verbose_name="Instagram edited"
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="edited_by_twitter",
            field=models.BooleanField(
                blank=True, default=False, verbose_name="Twitter edited"
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="edited_by_user",
            field=models.BooleanField(default=False, verbose_name="User edited"),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="google_isPlusUser",
            field=models.BooleanField(
                blank=True, default=False, null=True, verbose_name="Google Plus"
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="google_verified",
            field=models.BooleanField(
                blank=True, default=False, null=True, verbose_name="Google Verified"
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="image_avatar",
            field=models.CharField(
                blank=True,
                choices=[
                    ("socials", "Socials"),
                    ("predef", "Predefined"),
                    ("custom", "Custom"),
                ],
                default="socials",
                max_length=100,
                verbose_name="Avatar Picture",
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="image_avatar_predef",
            field=models.CharField(
                blank=True,
                choices=[
                    ("avatar1.png", "Male 1"),
                    ("avatar2.png", "Male 2"),
                    ("avatar3.png", "Female 1"),
                    ("avatar4.png", "Male 3"),
                    ("avatar5.png", "Male 4"),
                    ("avatar6.png", "Male 5"),
                    ("avatar7.png", "Male 6"),
                    ("avatar8.png", "Female 2"),
                ],
                default="avatar1.png",
                max_length=100,
                verbose_name="Predef Avatar Picture",
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="is_active",
            field=models.BooleanField(
                blank=True,
                default=True,
                help_text="Designates whether this user should be treated as active. Unselect this instead of deleting accounts.",
                null=True,
                verbose_name="Active",
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="is_staff",
            field=models.BooleanField(
                blank=True,
                default=False,
                help_text="Designates whether the user can log into this admin site.",
                null=True,
                verbose_name="Staff",
            ),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="twitter_verified",
            field=models.BooleanField(
                blank=True, default=False, null=True, verbose_name="Twitter Verified"
            ),
        ),
    ]



# Generated by Django 2.2.24 on 2021-11-30 18:37

import django_countries.fields
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("socialprofile", "0017_auto_20211129_2305"),
    ]

    operations = [
        migrations.AlterField(
            model_name="socialprofile",
            name="country",
            field=django_countries.fields.CountryField(
                blank=True, max_length=2, null=True
            ),
        ),
    ]



"""Unit Tests for the socialprofile module tags """

# pylint: disable=R0904, C0103

from django.conf import settings
from django.template import Context, Template
from django.test import TestCase
from social_core.backends.utils import load_backends


class TestSocialProfileTags(TestCase):
    TEMPLATE_BACKEND_NAME = Template(
        "{% load socialprofile_tags %}{% for sublist in available_backends|social_backends %}{% for name, backend in sublist %}"
        "{{ name }} "
        "{% endfor %}{% endfor %}"
    )
    TEMPLATE_BACKEND_ICON_NAME = Template(
        "{% load socialprofile_tags %}{% for sublist in available_backends|social_backends %}{% for name, backend in sublist %}"
        "{{ name|icon_name }} "
        "{% endfor %}{% endfor %}"
    )
    TEMPLATE_BACKEND_CLASS = Template(
        "{% load socialprofile_tags %}"
        "{% for sublist in available_backends|social_backends %}"
        "{{ sublist }}"
        "{% for name, backend in sublist %}"
        "{{ name }} {{ backend }}"
        "{% endfor %}"
        "{% endfor %}"
    )
    # TEMPLATE_ASSOCIATED = Template(
    #     "{% load socialprofile_tags %}{% available_backends code as country %}{{ country }}"
    # )
    # TEMPLATE_COUNTRY = Template(
    #     "{% load socialprofile_tags %}{% available_backends code as country %}{{ country }}"
    #     "{% load socialprofile_tags %}{% social_backends code as country %}{{ country }}"
    # )
    # TEMPLATE_NAME = Template(
    #     "{% load socialprofile_tags %}{% get_country code as country %}{{ country.name }}"
    # )

    def test_backend_name(self):
        rendered = self.TEMPLATE_BACKEND_NAME.render(
            Context(
                {"available_backends": load_backends(settings.AUTHENTICATION_BACKENDS)}
            )
        )
        self.assertEqual(rendered, "google-oauth2 twitter ")

    def test_backend_icon_name(self):
        rendered = self.TEMPLATE_BACKEND_ICON_NAME.render(
            Context(
                {"available_backends": load_backends(settings.AUTHENTICATION_BACKENDS)}
            )
        )
        self.assertEqual(rendered, "google twitter ")

    # def test_country_name(self):
    #     with translation.override("pt-BR"):
    #         rendered = self.TEMPLATE_NAME.render(Context({"code": "BR"}))
    #     self.assertEqual(rendered, "Brasil")
    #
    # def test_wrong_code(self):
    #     rendered = self.TEMPLATE_COUNTRY.render(Context({"code": "XX"}))
    #     self.assertEqual(rendered, "XX")
    #
    #     rendered = self.TEMPLATE_NAME.render(Context({"code": "XX"}))
    #     self.assertEqual(rendered, "")



# Generated by Django 2.2.25 on 2021-12-16 16:45

import image_cropping.fields
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("socialprofile", "0022_auto_20211215_2357"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="socialprofile",
            name="edited_by_provider",
        ),
        migrations.AddField(
            model_name="socialprofile",
            name="edited_by_facebook",
            field=models.NullBooleanField(
                default=False, verbose_name="Facebook edited"
            ),
        ),
        migrations.AddField(
            model_name="socialprofile",
            name="edited_by_google",
            field=models.NullBooleanField(default=False, verbose_name="Google edited"),
        ),
        migrations.AddField(
            model_name="socialprofile",
            name="edited_by_instagram",
            field=models.NullBooleanField(
                default=False, verbose_name="Instagram edited"
            ),
        ),
        migrations.AddField(
            model_name="socialprofile",
            name="edited_by_twitter",
            field=models.NullBooleanField(default=False, verbose_name="Twitter edited"),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="cropping",
            field=image_cropping.fields.ImageRatioField(
                "image",
                "120x100",
                adapt_rotation=False,
                allow_fullsize=False,
                free_crop=False,
                help_text=None,
                hide_image_field=False,
                size_warning=True,
                verbose_name="cropping",
            ),
        ),
    ]



# Generated by Django 2.2.25 on 2021-12-14 19:19

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("socialprofile", "0020_remove_socialprofile_last_accessed"),
    ]

    operations = [
        migrations.AddField(
            model_name="socialprofile",
            name="facebook_avatar",
            field=models.URLField(
                blank=True, max_length=500, null=True, verbose_name="Facebook Avatar"
            ),
        ),
        migrations.AddField(
            model_name="socialprofile",
            name="google_avatar",
            field=models.URLField(
                blank=True, max_length=500, null=True, verbose_name="Google Avatar"
            ),
        ),
        migrations.AddField(
            model_name="socialprofile",
            name="instagram_avatar",
            field=models.URLField(
                blank=True, max_length=500, null=True, verbose_name="Instagram Avatar"
            ),
        ),
        migrations.AddField(
            model_name="socialprofile",
            name="twitter_avatar",
            field=models.URLField(
                blank=True, max_length=500, null=True, verbose_name="Twitter Avatar"
            ),
        ),
    ]



"""Test's settings"""

DEBUG = False

PASSWORD_HASHERS = ["django.contrib.auth.hashers.MD5PasswordHasher"]

SECRET_KEY = "NOTASECRET"

ALLOWED_HOSTS = ["*"]

DATABASES = {"default": {"ENGINE": "django.db.backends.sqlite3"}}

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
                "django.template.context_processors.media",
                "social_django.context_processors.backends",  # Need By SocialProfile
                "social_django.context_processors.login_redirect",  # Need By SocialProfile
            ],
        },
    },
]

ROOT_URLCONF = "socialprofile.urls"

USE_TZ = True
LANGUAGE_CODE = "en"
USE_I18N = True

STATIC_URL = "/static/"

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.sites",
    "django.contrib.staticfiles",
    "django_errors",  # Needed by Wrap Errors
    "axes",  # Needed by SocialProfile [Monitor Logins]
    "user_sessions",  # Needed by SocialProfile [One-Time-Password]
    "django_otp",  # Needed by SocialProfile [One-Time-Password]
    "django_otp.plugins.otp_static",  # Needed by SocialProfile [One-Time-Password]
    "django_otp.plugins.otp_totp",  # Needed by SocialProfile [One-Time-Password]
    "two_factor",  # Needed by SocialProfile [One-Time-Password]
    "social_django",  # Needed by SocialProfile [OAuth]
    "rest_framework.authtoken",  # Needed by SocialProfile [Token]
    "rest_framework",  # Needed by SocialProfile
    "oauth2_provider",  # Needed by SocialProfile [OAuth2 Token]
    "corsheaders",  # Needed by SocialProfile [OAuth2 Token]
    "crispy_forms",  # Needed by SocialProfile [Edit Form]
    "django_countries",  # Needed by SocialProfile [Edit Form]
    "socialprofile",
]

# Social Profile
### Custom Social Profile User

MIDDLEWARE = (
    "django.middleware.security.SecurityMiddleware",
    # 'django.contrib.sessions.middleware.SessionMiddleware',    # Need Commente-Out By user_sessions
    "user_sessions.middleware.SessionMiddleware",  # Need By SocialProfile
    "django.middleware.cache.UpdateCacheMiddleware",
    "htmlmin.middleware.HtmlMinifyMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "oauth2_provider.middleware.OAuth2TokenMiddleware",  # Need By SocialProfile
    "django.middleware.locale.LocaleMiddleware",
    "django_otp.middleware.OTPMiddleware",  # Need By SocialProfile
    "corsheaders.middleware.CorsMiddleware",  # Need By SocialProfile
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "django.middleware.cache.FetchFromCacheMiddleware",
    "htmlmin.middleware.MarkRequestMiddleware",
    "social_django.middleware.SocialAuthExceptionMiddleware",
    "axes.middleware.AxesMiddleware",
)

# ------------------------
# Need by Social Profile / user_sessions
SESSION_ENGINE = "user_sessions.backends.db"
SESSION_SERIALIZER = "django.contrib.sessions.serializers.PickleSerializer"

# Core Authentication Settings
LOGIN_URL = "/sp/select/"
LOGIN_ERROR_URL = "/sp/select/"
LOGIN_REDIRECT_URL = "/sp/"
DEFAULT_RETURNTO_PATH = "/sp/"

# Custom user's username settings
SP_SET_USERNAME = True

# Social Auth Redirects Settings
SOCIAL_AUTH_LOGIN_REDIRECT_URL = "/sp/"
SOCIAL_AUTH_LOGIN_ERROR_URL = "/sp/login-error/"
SOCIAL_AUTH_LOGIN_URL = "/sp/select/"
SOCIAL_AUTH_NEW_USER_REDIRECT_URL = "/sp/new-profile/"
SOCIAL_AUTH_NEW_ASSOCIATION_REDIRECT_URL = "/sp/new-association/"

# Custom Social Profile User
AUTH_USER_MODEL = "socialprofile.socialprofile"
SOCIAL_AUTH_USER_MODEL = "socialprofile.socialprofile"

SOCIAL_AUTH_RAISE_EXCEPTIONS = True

SOCIAL_AUTH_STRATEGY = "social_django.strategy.DjangoStrategy"
SOCIAL_AUTH_STORAGE = "social_django.models.DjangoStorage"

SOCIAL_AUTH_PIPELINE = (
    "social_core.pipeline.social_auth.social_details",
    "social_core.pipeline.social_auth.social_uid",
    "social_core.pipeline.social_auth.auth_allowed",
    "social_core.pipeline.social_auth.social_user",
    "social_core.pipeline.user.get_username",
    "social_core.pipeline.mail.mail_validation",
    "social_core.pipeline.social_auth.associate_by_email",
    "social_core.pipeline.user.create_user",
    "social_core.pipeline.social_auth.associate_user",
    "social_core.pipeline.debug.debug",
    "social_core.pipeline.social_auth.load_extra_data",
    "social_core.pipeline.user.user_details",
    "social_core.pipeline.debug.debug",
    "socialprofile.pipeline.socialprofile_extra_values",
)

SOCIAL_AUTH_ALWAYS_ASSOCIATE = True
SOCIAL_AUTH_PROTECTED_USER_FIELDS = [
    "username",
    "email",
    "first_name",
    "last_name",
]
# SOCIAL_AUTH_IMMUTABLE_USER_FIELDS = ["username", "email", "first_name", "last_name", ]
SOCIAL_AUTH_REVOKE_TOKENS_ON_DISCONNECT = True


AUTHENTICATION_BACKENDS = (
    # 'social_core.backends.facebook.FacebookAppOAuth2',
    # 'social_core.backends.facebook.FacebookOAuth2',
    "social_core.backends.google.GoogleOAuth2",
    # 'social_core.backends.instagram.InstagramOAuth2',
    # 'social_core.backends.linkedin.LinkedinOAuth2',
    # 'social_core.backends.live.LiveOAuth2',
    "social_core.backends.twitter.TwitterOAuth",
    # 'social_core.backends.yahoo.YahooOAuth',
    # 'social_core.backends.vimeo.VimeoOAuth1',
    "social_core.backends.email.EmailAuth",
    "axes.backends.AxesBackend",
    "django.contrib.auth.backends.ModelBackend",
)

# ToDo: test for tests/test_user.py::EmailUserAdminTest::
OAUTH2_PROVIDER = {
    "OAUTH2_VALIDATOR_CLASS": "socialprofile.validators.AxesOAuth2Validator",
    "SCOPES": {"read": "Read scope", "write": "Write scope"},
}

# SOCIAL_AUTH_GOOGLE_OAUTH2_KEY = ''
# SOCIAL_AUTH_GOOGLE_OAUTH2_SECRET = ''
# SOCIAL_AUTH_GOOGLE_OAUTH2_SCOPE = [
#     "https://www.googleapis.com/auth/plus.login",
#     "https://www.googleapis.com/auth/user.birthday.read",
#     "https://www.googleapis.com/auth/user.gender.read"
# ]

# AUTHENTICATION_BACKENDS = (
#     # 'django.contrib.auth.backends.ModelBackend',  # Comment if you want DISABLE Login Form (Password)
#     "social.backends.amazon.AmazonOAuth2",
#     "social.backends.angel.AngelOAuth2",
#     "social.backends.aol.AOLOpenId",
#     "social.backends.appsfuel.AppsfuelOAuth2",
#     "social.backends.beats.BeatsOAuth2",
#     "social.backends.behance.BehanceOAuth2",
#     "social.backends.belgiumeid.BelgiumEIDOpenId",
#     "social.backends.bitbucket.BitbucketOAuth",
#     "social.backends.box.BoxOAuth2",
#     "social.backends.clef.ClefOAuth2",
#     "social.backends.coinbase.CoinbaseOAuth2",
#     # 'social.backends.coursera.CourseraOAuth2',
#     "social.backends.dailymotion.DailymotionOAuth2",
#     "social.backends.disqus.DisqusOAuth2",
#     "social.backends.douban.DoubanOAuth2",
#     # 'social.backends.dropbox.DropboxOAuth',
#     "social.backends.dropbox.DropboxOAuth2",
#     # 'social.backends.eveonline.EVEOnlineOAuth2',
#     "social.backends.evernote.EvernoteSandboxOAuth",
#     "social.backends.facebook.FacebookAppOAuth2",
#     "social.backends.facebook.FacebookOAuth2",
#     "social.backends.fedora.FedoraOpenId",
#     "social.backends.fitbit.FitbitOAuth",
#     "social.backends.flickr.FlickrOAuth",
#     "social.backends.foursquare.FoursquareOAuth2",
#     "social.backends.github.GithubOAuth2",
#     # 'social.backends.google.GoogleOAuth',
#     "social.backends.google.GoogleOAuth2",
#     # 'social.backends.google.GoogleOpenId',
#     # 'social.backends.google.GooglePlusAuth',
#     # 'social.backends.google.GoogleOpenIdConnect',
#     "social.backends.instagram.InstagramOAuth2",
#     "social.backends.jawbone.JawboneOAuth2",
#     "social.backends.kakao.KakaoOAuth2",
#     "social.backends.linkedin.LinkedinOAuth",
#     "social.backends.linkedin.LinkedinOAuth2",
#     "social.backends.live.LiveOAuth2",
#     "social.backends.livejournal.LiveJournalOpenId",
#     "social.backends.mailru.MailruOAuth2",
#     "social.backends.mendeley.MendeleyOAuth",
#     "social.backends.mendeley.MendeleyOAuth2",
#     # 'social.backends.mineid.MineIDOAuth2',
#     "social.backends.mixcloud.MixcloudOAuth2",
#     # 'social.backends.nationbuilder.NationBuilderOAuth2',
#     "social.backends.odnoklassniki.OdnoklassnikiOAuth2",
#     "social.backends.open_id.OpenIdAuth",
#     "social.backends.openstreetmap.OpenStreetMapOAuth",
#     "social.backends.persona.PersonaAuth",
#     "social.backends.podio.PodioOAuth2",
#     "social.backends.rdio.RdioOAuth1",
#     "social.backends.rdio.RdioOAuth2",
#     "social.backends.readability.ReadabilityOAuth",
#     "social.backends.reddit.RedditOAuth2",
#     "social.backends.runkeeper.RunKeeperOAuth2",
#     "social.backends.skyrock.SkyrockOAuth",
#     "social.backends.soundcloud.SoundcloudOAuth2",
#     "social.backends.spotify.SpotifyOAuth2",
#     "social.backends.stackoverflow.StackoverflowOAuth2",
#     "social.backends.steam.SteamOpenId",
#     "social.backends.stocktwits.StocktwitsOAuth2",
#     "social.backends.stripe.StripeOAuth2",
#     "social.backends.suse.OpenSUSEOpenId",
#     "social.backends.thisismyjam.ThisIsMyJamOAuth1",
#     "social.backends.trello.TrelloOAuth",
#     "social.backends.tripit.TripItOAuth",
#     "social.backends.tumblr.TumblrOAuth",
#     "social.backends.twilio.TwilioAuth",
#     "social.backends.twitter.TwitterOAuth",
#     "social.backends.vk.VKOAuth2",
#     "social.backends.weibo.WeiboOAuth2",
#     # 'social.backends.wunderlist.WunderlistOAuth2',
#     "social.backends.xing.XingOAuth",
#     "social.backends.yahoo.YahooOAuth",
#     "social.backends.yahoo.YahooOpenId",
#     "social.backends.yammer.YammerOAuth2",
#     "social.backends.yandex.YandexOAuth2",
#     "social.backends.vimeo.VimeoOAuth1",
#     "social.backends.lastfm.LastFmAuth",
#     "social.backends.moves.MovesOAuth2",
#     # 'social.backends.vend.VendOAuth2',
#     "social.backends.email.EmailAuth",
#     "social.backends.username.UsernameAuth",
# )



"""Custom Widget to enable use of HTML5 Form element types"""

from django.forms.widgets import DateInput, DateTimeInput, TextInput, TimeInput


class H5EmailInput(TextInput):
    """HTML5 Email Input Type"""

    input_type = "email"


class H5NumberInput(TextInput):
    """HTML5 Number Input Type"""

    input_type = "number"


class H5TelephoneInput(TextInput):
    """HTML5 Telephone Number Input Type"""

    input_type = "tel"


class H5DateInput(DateInput):
    """HTML5 Date Input Type"""

    input_type = "date"


class H5DateTimeInput(DateTimeInput):
    """HTML5 DateTime Input Type"""

    input_type = "datetime"


class H5TimeInput(TimeInput):
    """HTML5 Time Input Type"""

    input_type = "time"



from axes.helpers import get_client_ip_address, get_client_user_agent
from django.contrib.auth import authenticate
from django.http import HttpRequest, QueryDict
from oauth2_provider.oauth2_validators import OAuth2Validator


class AxesOAuth2Validator(OAuth2Validator):
    def validate_user(self, username, password, client, request, *args, **kwargs):
        """
        Check username and password correspond to a valid and active User
        Set defaults for necessary request object attributes for Axes compatibility.
        The ``request`` argument is not a Django ``HttpRequest`` object.
        """
        _request = request
        if request and not isinstance(request, HttpRequest):
            request = HttpRequest()
            request.uri = _request.uri
            request.method = request.http_method = _request.http_method
            request.META = request.headers = _request.headers
            request._params = _request._params
            request.decoded_body = _request.decoded_body
            request.axes_ip_address = get_client_ip_address(request)
            request.axes_user_agent = get_client_user_agent(request)
            body = QueryDict(str(_request.body), mutable=True)
            if request.method == "GET":
                request.GET = body
            elif request.method == "POST":
                request.POST = body
        user = authenticate(request=request, username=username, password=password)
        if user is not None and user.is_active:
            request = _request
            request.user = user
            return True
        return False



# Generated by Django 3.2.12 on 2022-03-17 09:59

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("oauth2_provider", "0005_auto_20211222_2352"),
        ("socialprofile", "0024_auto_20220111_0008"),
    ]

    operations = [
        migrations.CreateModel(
            name="ProxyIDToken",
            fields=[],
            options={
                "verbose_name": "Token: id token",
                "verbose_name_plural": "Token: id tokens",
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("oauth2_provider.idtoken",),
        ),
        migrations.AddField(
            model_name="socialprofile",
            name="live_avatar",
            field=models.URLField(
                blank=True, max_length=500, null=True, verbose_name="Live Avatar"
            ),
        ),
        migrations.AddField(
            model_name="socialprofile",
            name="live_language",
            field=models.CharField(
                blank=True, max_length=10, null=True, verbose_name="Live Language"
            ),
        ),
        migrations.AddField(
            model_name="socialprofile",
            name="live_url",
            field=models.URLField(
                blank=True, max_length=500, null=True, verbose_name="Live Profile"
            ),
        ),
        migrations.AddField(
            model_name="socialprofile",
            name="live_username",
            field=models.CharField(
                blank=True, max_length=30, verbose_name="Live Username"
            ),
        ),
        migrations.AddField(
            model_name="socialprofile",
            name="live_verified",
            field=models.BooleanField(
                blank=True, default=False, null=True, verbose_name="Live Verified"
            ),
        ),
    ]



"""
Alternative implementation of Django's authentication User model, which allows to authenticate against the OAuth.
This alternative implementation is activated by setting ``AUTH_USER_MODEL = 'socialprofile.SocialProfile'`` in
settings.py, otherwise the default Django or another implementation is used.
"""

import logging

from django.conf import settings
from django.contrib.auth.models import (
    AbstractBaseUser,
    BaseUserManager,
    PermissionsMixin,
)
from django.contrib.auth.validators import ASCIIUsernameValidator
from django.core.mail import send_mail
from django.core.validators import RegexValidator
from django.db import models
from django.templatetags.static import static
from django.urls import reverse
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from django_countries.fields import CountryField
from image_cropping.fields import ImageCropField, ImageRatioField

LOGGER = logging.getLogger(name="socialprofile.models")


class IntegerRangeField(models.IntegerField):
    def __init__(
        self, verbose_name=None, name=None, min_value=None, max_value=None, **kwargs
    ):
        self.min_value, self.max_value = min_value, max_value
        models.IntegerField.__init__(self, verbose_name, name, **kwargs)

    def formfield(self, **kwargs):
        defaults = {"min_value": self.min_value, "max_value": self.max_value}
        defaults.update(kwargs)
        return super().formfield(**defaults)


class SocialProfileManager(BaseUserManager):
    use_in_migrations = True

    def get_by_natural_key(self, username):
        try:
            return self.get(username=username)
        except self.model.DoesNotExist:
            return self.get(is_active=True, email=username)

    def _create_user(
        self,
        email,
        password,
        username=None,
        is_staff=False,
        is_superuser=False,
        is_active=True,
        **extra_fields,
    ):
        """
        Creates and saves a User with the given email and password.
        :param str email: user email
        :param str password: user password
        :param str username: user's name
        :param bool is_staff: whether user staff or not
        :param bool is_superuser: whether user admin or not
        :param bool is_active: user state active/disabled
        :return socialprofile.models.SocialProfile user: user
        :raise ValueError: email is not set
        """
        now = timezone.now()
        if not email:
            raise ValueError(_("The given email must be set"))
        email = self.normalize_email(email)
        user = self.model(
            email=email,
            # username=username,
            is_staff=is_staff,
            is_superuser=is_superuser,
            is_active=is_active,
            last_login=now,
            date_joined=now,
            **extra_fields,
        )
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_user(
        self,
        email,
        password=None,
        username=None,
        is_staff=False,
        is_superuser=False,
        is_active=True,
        **extra_fields,
    ):
        """
        Create and save a SocialProfile with the given email and password.
        :param str email: user email
        :param str password: user password
        :param str username: user's name
        :param bool is_staff: whether user staff or not
        :param bool is_superuser: whether user admin or not
        :param bool is_active: user state active/disabled
        :return socialprofile.models.SocialProfile user: regular user
        """
        # is_staff = extra_fields.pop("is_staff", False)
        # is_superuser = extra_fields.pop("is_superuser", False)
        return self._create_user(
            email, password, username, is_staff, is_superuser, is_active, **extra_fields
        )

    def create_superuser(
        self,
        email,
        password=None,
        username=None,
        is_staff=True,
        is_superuser=True,
        is_active=True,
        **extra_fields,
    ):
        return self._create_user(
            email, password, username, is_staff, is_superuser, is_active, **extra_fields
        )


class AbstractSocialProfile(AbstractBaseUser, PermissionsMixin):
    """
    Alternative implementation of Django's User model allowing to authenticate against the OAuth.

    Inherits from both the AbstractBaseUser and PermissionMixin.
    The following attributes are inherited from the superclasses:
        * password
        * last_login
        * is_superuser
    """

    username_validator = ASCIIUsernameValidator()

    GENDER_CHOICES = (
        ("male", _("Male")),
        ("female", _("Female")),
        ("unknown", _("Unknown")),
    )

    AVATAR_CHOICES = (
        ("socials", _("Socials")),
        ("predef", _("Predefined")),
        ("custom", _("Custom")),
    )

    AVATAR_IMG_CHOICES = (
        ("avatar1.png", _("Male 1")),
        ("avatar2.png", _("Male 2")),
        ("avatar3.png", _("Female 1")),
        ("avatar4.png", _("Male 3")),
        ("avatar5.png", _("Male 4")),
        ("avatar6.png", _("Male 5")),
        ("avatar7.png", _("Male 6")),
        ("avatar8.png", _("Female 2")),
    )

    BOOLEAN_YN = (
        (True, _("Yes")),
        (False, _("No")),
    )

    USERNAME_FIELD = "username"
    REQUIRED_FIELDS = ["email"]

    username = models.CharField(
        _("Username"),
        max_length=30,
        unique=True,
        blank=False,
        help_text=_(
            "Required. 30 characters or fewer. Letters, digits and @/./+/-/_ only"
        ),
        validators=[
            RegexValidator(
                r"^[\w.@+-]+$",
                _(
                    "Enter a valid username. This value may contain "
                    "30 characters or fewer. Only letters, numbers "
                    "and @/./+/-/_ characters"
                ),
            ),
        ],
        error_messages={
            "unique": _("A user with that username already exists."),
        },
    )
    first_name = models.CharField(
        verbose_name=_("First Name"), max_length=30, blank=True
    )
    last_name = models.CharField(verbose_name=_("Last Name"), max_length=30, blank=True)
    gender = models.CharField(
        verbose_name=_("Gender"),
        max_length=10,
        null=False,
        blank=None,
        default="unknown",
        choices=GENDER_CHOICES,
    )
    email = models.EmailField(
        verbose_name=_("Email Address"), max_length=254, null=False, blank=False
    )
    is_staff = models.BooleanField(
        verbose_name=_("Staff"),
        default=False,
        null=True,
        blank=True,
        help_text=_("Designates whether the user can log into this admin site."),
    )
    is_active = models.BooleanField(
        verbose_name=_("Active"),
        default=True,
        null=True,
        blank=True,
        help_text=_(
            "Designates whether this user should be treated as active. Unselect this "
            "instead of deleting accounts."
        ),
    )
    date_of_birth = models.DateField(
        verbose_name=_("Date of Birth"), null=True, blank=True
    )
    date_joined = models.DateTimeField(verbose_name=_("Date Joined"), auto_now_add=True)
    date_modified = models.DateTimeField(verbose_name=_("Date Updated"), auto_now=True)

    # status = models.ForeignKey(MembershipStatus, on_delete=models.SET_NULL, null=True, default=1)
    country = CountryField(_("Country"), null=True, blank=True)

    city = models.CharField(
        verbose_name=_("City"), max_length=255, null=True, blank=True
    )
    address = models.CharField(
        verbose_name=_("Address"), max_length=255, null=True, blank=True
    )
    postalcode = models.CharField(
        verbose_name=_("Postal Code"), max_length=255, null=True, blank=True
    )

    url = models.URLField(
        verbose_name=_("Homepage"),
        help_text=_("Where can we find out more about you?"),
        max_length=500,
        null=True,
        blank=True,
    )
    image_url = models.URLField(
        verbose_name=_("Avatar Picture"), max_length=500, null=True, blank=True
    )

    image_avatar = models.CharField(
        verbose_name=_("Avatar Picture"),
        max_length=100,
        null=False,
        blank=True,
        default="socials",
        choices=AVATAR_CHOICES,
    )
    image_avatar_predef = models.CharField(
        verbose_name=_("Predef Avatar Picture"),
        max_length=100,
        null=False,
        blank=True,
        default="avatar1.png",
        choices=AVATAR_IMG_CHOICES,
    )
    # Add Images
    # image = models.ImageField(upload_to="user/", null=True, blank=True)
    image = ImageCropField(upload_to="user/", null=True, blank=True)
    cropping = ImageRatioField(
        "image", "120x100", allow_fullsize=False, size_warning=True
    )
    cropping_free = ImageRatioField(
        "image", "300x300", free_crop=True, size_warning=True
    )
    description = models.TextField(
        verbose_name=_("Description"),
        help_text=_("Tell us about yourself!"),
        null=True,
        blank=True,
    )

    # Add Contact info
    # max_length=15->'^\+?1?\d{9,15}$'
    phone_regex = RegexValidator(
        regex=r"^\+\d{8,15}$",
        message=_(
            "Phone number must be entered in the format: '+999999999'. Up to 15 digits allowed."
        ),
    )
    phone_number = models.CharField(
        verbose_name=_("Phone Number"),
        validators=[phone_regex],
        max_length=16,
        null=True,
        blank=True,
    )

    # Add for Work Infos
    sort = IntegerRangeField(
        verbose_name=_("Sort order"),
        default=1,
        min_value=1,
        max_value=100000000,
        null=True,
        blank=True,
    )
    visible = models.BooleanField(
        verbose_name=_("Public"),
        default=True,
        null=False,
        blank=False,
        choices=BOOLEAN_YN,
        help_text=_(
            "Designates whether this user should be visible by all. Select 'no' this make "
            "your account private!"
        ),
    )

    company = models.CharField(
        verbose_name=_("Company"), max_length=255, null=True, blank=True
    )
    title = models.CharField(
        verbose_name=_("Title"),
        default=_("Member"),
        max_length=500,
        null=True,
        blank=True,
    )
    role = models.CharField(
        verbose_name=_("Role"), max_length=500, null=True, blank=True
    )
    function_01 = models.CharField(max_length=200, null=True, blank=True)
    function_02 = models.CharField(max_length=200, null=True, blank=True)
    function_03 = models.CharField(max_length=200, null=True, blank=True)
    function_04 = models.CharField(max_length=200, null=True, blank=True)
    function_05 = models.CharField(max_length=200, null=True, blank=True)
    function_06 = models.CharField(max_length=200, null=True, blank=True)
    function_07 = models.CharField(max_length=200, null=True, blank=True)
    function_08 = models.CharField(max_length=200, null=True, blank=True)
    function_09 = models.CharField(max_length=200, null=True, blank=True)
    function_10 = models.CharField(max_length=200, null=True, blank=True)

    edited_by_user = models.BooleanField(
        verbose_name=_("User edited"), default=False, null=False, blank=False
    )
    edited_by_google = models.BooleanField(
        verbose_name=_("Google edited"), default=False, null=False, blank=True
    )
    edited_by_twitter = models.BooleanField(
        verbose_name=_("Twitter edited"), default=False, null=False, blank=True
    )
    edited_by_facebook = models.BooleanField(
        verbose_name=_("Facebook edited"), default=False, null=False, blank=True
    )
    edited_by_instagram = models.BooleanField(
        verbose_name=_("Instagram edited"), default=False, null=False, blank=True
    )
    edited_by_live = models.BooleanField(
        verbose_name=_("Live edited"), default=False, null=False, blank=True
    )

    # Add Info retrieved by Google
    google_email = models.EmailField(
        verbose_name=_("Google's Email"), max_length=254, null=True, blank=True
    )
    google_username = models.CharField(
        _("Google Username"),
        max_length=30,
        # unique=True,
        blank=True,
    )
    google_isPlusUser = models.BooleanField(
        verbose_name=_("Google Plus"), default=False, null=True, blank=True
    )
    google_url = models.URLField(
        verbose_name=_("Google Plus Page"), max_length=500, null=True, blank=True
    )
    google_circledByCount = models.IntegerField(
        verbose_name=_("Google Circle byCount"), default=0, null=True, blank=True
    )
    google_language = models.CharField(
        verbose_name=_("Google Language"), max_length=10, null=True, blank=True
    )
    google_kind = models.CharField(
        verbose_name=_("Google Kind"), max_length=15, null=True, blank=True
    )
    google_verified = models.BooleanField(
        verbose_name=_("Google Verified"), default=False, null=True, blank=True
    )
    google_avatar = models.URLField(
        verbose_name=_("Google Avatar"), max_length=500, null=True, blank=True
    )

    # Add Info retrieved by Twitter
    twitter_email = models.EmailField(
        verbose_name=_("Twitter's Email"), max_length=254, null=True, blank=True
    )
    twitter_username = models.CharField(
        _("Twitter Username"),
        max_length=30,
        # unique=True,
        blank=True,
    )
    twitter_language = models.CharField(
        verbose_name=_("Twitter Language"), max_length=10, null=True, blank=True
    )
    twitter_verified = models.BooleanField(
        verbose_name=_("Twitter Verified"), default=False, null=True, blank=True
    )
    twitter_url = models.URLField(
        verbose_name=_("Twitter Profile"),
        max_length=500,
        null=True,
        blank=True,
    )
    twitter_avatar = models.URLField(
        verbose_name=_("Twitter Avatar"), max_length=500, null=True, blank=True
    )

    # Add Info retrieved by Facebook
    facebook_email = models.EmailField(
        verbose_name=_("Facebook's Email"), max_length=254, null=True, blank=True
    )
    facebook_username = models.CharField(
        _("Facebook Username"),
        max_length=30,
        # unique=True,
        blank=True,
    )
    facebook_url = models.URLField(
        verbose_name=_("Facebook Profile"),
        max_length=500,
        null=True,
        blank=True,
    )
    facebook_avatar = models.URLField(
        verbose_name=_("Facebook Avatar"), max_length=500, null=True, blank=True
    )

    # Add Info retrieved by Instagram
    instagram_email = models.EmailField(
        verbose_name=_("Instagram's Email"), max_length=254, null=True, blank=True
    )
    instagram_username = models.CharField(
        _("Instagram Username"),
        max_length=30,
        # unique=True,
        blank=True,
    )
    instagram_url = models.URLField(
        verbose_name=_("Instagram Profile"),
        max_length=500,
        null=True,
        blank=True,
    )
    instagram_avatar = models.URLField(
        verbose_name=_("Instagram Avatar"), max_length=500, null=True, blank=True
    )

    # Add Info retrieved by Live
    live_email = models.EmailField(
        verbose_name=_("Live's Email"), max_length=254, null=True, blank=True
    )
    live_username = models.CharField(
        _("Live Username"),
        max_length=30,
        # unique=True,
        blank=True,
    )
    live_language = models.CharField(
        verbose_name=_("Live Language"), max_length=10, null=True, blank=True
    )
    live_verified = models.BooleanField(
        verbose_name=_("Live Verified"), default=False, null=True, blank=True
    )
    live_url = models.URLField(
        verbose_name=_("Live Profile"),
        max_length=500,
        null=True,
        blank=True,
    )
    live_avatar = models.URLField(
        verbose_name=_("Live Avatar"), max_length=500, null=True, blank=True
    )

    objects = SocialProfileManager()

    class Meta:
        verbose_name = _("Social Profile")
        verbose_name_plural = _("Social Profiles")
        ordering = ["username"]
        proxy = False
        abstract = True

    # def save(self, *args, **kwargs):
    # if not self.username:
    # self.username = self.get_email()
    # super(AbstractSocialProfile, self).save(*args, **kwargs)

    def __str__(self):
        return self.get_email()

    def get_absolute_url(self):
        return reverse("sp_profile_other_view_page", args=[self.username])

    def get_username(self):
        """Return the username"""
        custom_setting = getattr(settings, "SP_SET_USERNAME", False)
        if self.is_staff or custom_setting:
            return self.username
        return self.email or "<anonymous>"

    def get_email(self):
        """Return the email"""
        return self.email

    def get_full_name(self):
        """
        Returns the first_name plus the last_name, with a space in between else email
        """
        full_name = f"{self.first_name} {self.last_name}"
        if len(full_name) > 1:
            return full_name.strip()
        return self.get_email()

    def get_short_name(self):
        """
        Returns the short name for the user.
        """
        if len(self.first_name) > 1:
            return self.first_name
        return self.get_email()

    def get_avatar_img(self):
        """
        Returns the short name for the user.
        """
        if self.image_avatar:
            return (
                "https://www.gravatar.com/avatar/00000000000000000000000000000000?d=mm"
            )

        if self.image_avatar == "socials":
            if self.google_avatar:
                return self.google_avatar
            elif self.twitter_avatar:
                return self.twitter_avatar
            elif self.facebook_avatar:
                return self.facebook_avatar
            elif self.instagram_avatar:
                return self.instagram_avatar
            elif self.live_avatar:
                return self.live_avatar
            else:
                return static(f"{self.image}")

        elif self.image_avatar == "predef":
            return static(f"socialprofile/img/avatar/{self.image_avatar_predef}")

        else:
            return static(f"{self.image}")

    def email_user(self, subject, message, from_email=None, **kwargs):
        """Send an email to this User."""
        send_mail(subject, message, from_email, [self.email], **kwargs)


class SocialProfile(AbstractSocialProfile):
    """
    Concrete class of AbstractSocialProfile.
    Use this if you don't need to extend SocialProfile.
    """

    class Meta(AbstractSocialProfile.Meta):  # noqa: D101
        swappable = "AUTH_USER_MODEL"


# def __str__(self):
#     return "{}".format(self.group.name)
#
# group = models.OneToOneField('auth.Group', unique=True, parent_link=True, on_delete=models.CASCADE)


# def validate_unique(self, exclude=self.id):
# """
# Since the email address is used as the primary identifier, we must ensure that it is
# unique. However, this can not be done on the field declaration since is only applies to
# active users. Inactive users can not login anyway, so we don't need a unique constraint
# for them.
# """
# super(SocialProfile, self).validate_unique(exclude)
# if self.email and get_user_model().objects.exclude(id=self.id).filter(is_active=True, email__exact=self.email).exists():
# msg = _("A customer with the e-mail address ‘{email}’ already exists.")
# raise ValidationError({'email': msg.format(email=self.email)})

# class UserDetails(models.Model):
# type = models.OneToOneField('SocialProfile')
# extra_info = models.CharField(max_length=600)

# def create_user_profile(sender, instance, created, **kwargs):
# """Creates a UserProfile Object Whenever a User Object is Created"""
# if created:
# SocialProfile.objects.create(user=instance)


# post_save.connect(create_user_profile, sender=settings.AUTH_USER_MODEL)



import image_cropping.fields
from django.db import migrations

import socialprofile.models


class Migration(migrations.Migration):
    dependencies = [
        ("socialprofile", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="socialprofile",
            name="image",
            field=image_cropping.fields.ImageCropField(upload_to="user/", blank=True),
        ),
        migrations.AlterField(
            model_name="socialprofile",
            name="sort",
            field=socialprofile.models.IntegerRangeField(
                default=1, verbose_name="Sort Order", blank=True
            ),
        ),
    ]



"""Unit Tests for the socialprofile module forms"""

import logging

from django.test import TestCase
from social_core.backends.facebook import FacebookOAuth2
from social_core.backends.google import GoogleOAuth2
from social_core.backends.twitter import TwitterOAuth

from socialprofile.models import SocialProfile
from socialprofile.pipeline import socialprofile_extra_values

LOGGER = logging.getLogger(name="socialprofile.test_pipeline")


class SocialProfilePipelineTestCase(TestCase):
    """Test Case for Social Profile Pipeline"""

    def setUp(self):
        """Set up common assets for tests"""
        LOGGER.debug("SocialProfile Pipeline Tests setUp")
        self.user1 = SocialProfile.objects.create_user(
            "user1@user1.com", "mypassword", "user1"
        )
        self.user1.description = "Test User 1"
        self.user1.gender = "other"
        self.user1.url = "http://test.url"

        self.user1.google_url = "http://test.url"
        self.user1.google_avatar = (
            "http://www.gravatar.com/avatar/00000000000000000000000000000000?d=mm"
        )

        self.user1.twitter_url = "http://test.url"
        self.user1.twitter_avatar = (
            "http://www.gravatar.com/avatar/00000000000000000000000000000000?d=mm"
        )

        self.user1.instagram_url = "http://test.url"
        self.user1.instagram_avatar = (
            "http://www.gravatar.com/avatar/00000000000000000000000000000000?d=mm"
        )

        self.user1.facebook_url = "http://test.url"
        self.user1.facebook_avatar = (
            "http://www.gravatar.com/avatar/00000000000000000000000000000000?d=mm"
        )
        self.user1.save()

    # self.sa1 = UserSocialAuth.objects.create(user=self.user1, provider='google-oauth2', uid='user1@user1.com')
    def test_socialprofile_pipeline_google(self):
        """Test editing executing pipeline methods in isolation for google"""
        LOGGER.debug("Test socialprofile pipeline Google")
        backend = GoogleOAuth2()
        response = {
            "name": {"familyName": "User 1", "givenName": "Test"},
            "gender": "other",
            "picture": "http://image-google.url",
            "occupation": "User Description",
            "url": "http://google.url",
        }

        socialprofile_extra_values(backend, {}, response, "1", self.user1)
        self.assertTrue(self.user1.edited_by_google)
        self.assertEqual(self.user1.description, "User Description")
        self.assertEqual(self.user1.gender, "other")
        self.assertEqual(self.user1.google_avatar, "http://image-google.url")
        self.assertEqual(self.user1.google_url, "http://google.url")

    def test_socialprofile_pipeline_facebook(self):
        """Test editing executing pipeline methods in isolation for facebook"""
        LOGGER.debug("Test socialprofile pipeline Facebook")
        backend = FacebookOAuth2()
        response = {
            "name": {"last_name": "User 1", "first_name": "Test"},
            "gender": "other",
            "picture": {"data": {"url": "http://image-facebook.url"}},
            "link": "http://facebook.url",
        }

        socialprofile_extra_values(backend, {}, response, "1", self.user1)
        self.assertTrue(self.user1.edited_by_facebook)
        self.assertEqual(self.user1.gender, "other")
        self.assertEqual(self.user1.facebook_avatar, "http://image-facebook.url")
        self.assertEqual(self.user1.facebook_url, "http://facebook.url")

    def test_socialprofile_pipeline_twitter(self):
        """Test editing executing pipeline methods in isolation for twitter"""
        LOGGER.debug("Test socialprofile pipeline Twitter")
        backend = TwitterOAuth()
        response = {
            "name": {"last_name": "User 1", "first_name": "Test"},
            "gender": "other",
            "profile_image_url_https": "http://image-twitter.url",
            "description": "User Description",
            "url": "http://twitter.url",
        }

        socialprofile_extra_values(backend, {}, response, "1", self.user1)
        self.assertTrue(self.user1.edited_by_twitter)
        self.assertEqual(self.user1.description, "User Description")
        self.assertEqual(self.user1.gender, "other")
        self.assertEqual(self.user1.twitter_avatar, "http://image-twitter.url")
        self.assertEqual(self.user1.twitter_url, "http://twitter.url")



# Generated by Django 4.2.16 on 2025-01-04 18:27

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("socialprofile", "0028_alter_proxysession_options"),
    ]

    operations = [
        migrations.AddField(
            model_name="socialprofile",
            name="facebook_email",
            field=models.EmailField(
                blank=True, max_length=254, null=True, verbose_name="Facebook's Email"
            ),
        ),
        migrations.AddField(
            model_name="socialprofile",
            name="google_email",
            field=models.EmailField(
                blank=True, max_length=254, null=True, verbose_name="Google's Email"
            ),
        ),
        migrations.AddField(
            model_name="socialprofile",
            name="instagram_email",
            field=models.EmailField(
                blank=True, max_length=254, null=True, verbose_name="Instagram's Email"
            ),
        ),
        migrations.AddField(
            model_name="socialprofile",
            name="live_email",
            field=models.EmailField(
                blank=True, max_length=254, null=True, verbose_name="Live's Email"
            ),
        ),
        migrations.AddField(
            model_name="socialprofile",
            name="twitter_email",
            field=models.EmailField(
                blank=True, max_length=254, null=True, verbose_name="Twitter's Email"
            ),
        ),
    ]



"""Unit Tests for the socialprofile module forms"""

# pylint: disable=R0904, C0103

import logging

from django.forms.models import model_to_dict
from django.test import TestCase
from social.apps.django_app.default.models import UserSocialAuth

from socialprofile.forms import SocialProfileForm
from socialprofile.models import SocialProfile

LOGGER = logging.getLogger(name="socialprofile.test_forms")


class SocialProfileFormTestCase(TestCase):
    """Test Case for Social Profile Forms"""

    def setUp(self):
        """Set up common assets for tests"""
        LOGGER.debug("SocialProfileForm Tests setUp")
        self.user1 = SocialProfile.objects.create_user(
            "user1@user1.com",
            username="user1",
        )
        self.user1.set_password("user1password")
        # todo: check because is not rendered as field inside forms
        self.user1.username = "user1"
        self.user1.gender = "other"
        self.user1.url = "http://test.com"
        self.user1.description = "Test User 1"
        self.user1.image_url = (
            "http://www.gravatar.com/avatar/00000000000000000000000000000000?d=mm"
        )
        self.user1.save()
        self.sa1 = UserSocialAuth.objects.create(
            user=self.user1, provider="google-oauth2", uid="user1@user1.com"
        )

    def test_socialprofile_form_view(self):
        """Test editing user profile data through form in isolation"""
        LOGGER.debug("Test socialprofile edit form")
        form = SocialProfileForm(instance=self.user1)
        form_html = form.as_p()
        LOGGER.debug(form_html)
        # self.assertInHTML(
        #     '<option value="other" selected="selected">Other</option>', form_html
        # )
        self.assertInHTML(
            '<input id="id_returnTo" name="returnTo" type="hidden" value="/" />',
            form_html,
        )
        self.assertInHTML(
            '<textarea cols="40" id="id_description" name="description" rows="10">Test User 1</textarea>',
            form_html,
        )
        # self.assertInHTML(
        #     '<input id="id_image_url" maxlength="500" name="image_url" type="url" value="http://www.gravatar.com/avatar/00000000000000000000000000000000?d=mm" />',
        #     form_html,
        # )
        # self.assertInHTML(
        #     '<input id="id_manually_edited" name="manually_edited" type="hidden" value="True" />',
        #     form_html,
        # )
        self.assertInHTML(
            '<input id="id_url" maxlength="500" name="url" type="url" value="http://test.com" />',
            form_html,
        )

    def test_socialprofile_form_update(self):
        """Test Form Update"""
        LOGGER.debug("Test socialprofile edit form")
        data = model_to_dict(self.user1)
        data["description"] = "new description"
        data["gender"] = "female"
        data["url"] = "http://new.url/"
        data["image_url"] = "http://new.image.url/"
        # data["country"] = 'en' # Raise error
        data["country"] = "IT"
        form = SocialProfileForm(data=data, instance=self.user1)
        if not form.is_valid():
            LOGGER.error(form.errors)
        self.assertTrue(form.is_valid())
        form.save()
        self.assertEqual(self.user1.description, "new description")
        self.assertEqual(self.user1.url, "http://new.url/")
        self.assertEqual(self.user1.gender, "female")
        # self.assertEqual(self.user1.image_url, "http://new.image.url/")

    def test_socialprofile_form_clean_desc(self):
        """Test Form Clean"""
        LOGGER.debug("Test socialprofile form clean desc")
        data = model_to_dict(self.user1)
        data["description"] = '<a href="http://bad.url">Bad Link</a>'
        data["gender"] = "female"
        form = SocialProfileForm(data=data, instance=self.user1)
        if not form.is_valid():
            LOGGER.error(form.errors)
        self.assertTrue(form.is_valid())
        form.save()
        self.assertEqual(self.user1.description, "Bad Link")


#
# class UserFormTestCase(TestCase):
#         """Tests for  User Form"""
#
#     def setUp(self):
#         """Set up common assets for tests"""
#         LOGGER.debug("UserForm Tests setUp")
#         self.user1 = User.objects.create_user('user1', 'user1@user1.com', 'user1password')
#
#     def test_user_form_create(self):
#         LOGGER.debug("Test user form create")
#         data = {
#         'username': 'user2',
#         'password': 'user2password',
#         'email': 'user@user2.com',
#         'first_name': 'user',
#         'last_name': '2',
#         'last_login': '2012-09-04 06:00',
#         'date_joined': '2012-09-04 06:00',
#         'manually_edited': False
#         }
#         form = UserForm(data)
#         self.assertTrue(form.is_valid())
#         form.save()
#         user2 = User.objects.get(username='user2')
#         self.assertEquals(user2.username, 'user2')
#
#     def test_user_form_view(self):
#         """Test editing user profile data through form in isolation"""
#         LOGGER.debug("Test user edit form")
#         form = UserForm(instance=self.user1)
#         form_html = form.as_p()
#         LOGGER.debug(form_html)
#         self.assertInHTML('<input id="id_username" maxlength="30" name="username" type="text" value="user1" />', form_html)
#         self.assertInHTML('<input id="id_first_name" maxlength="30" name="first_name" type="text" />', form_html)
#         self.assertInHTML('<input id="id_email" maxlength="75" name="email" type="email" value="user1@user1.com" />', form_html)
#         self.assertInHTML('<input id="id_last_name" maxlength="30" name="last_name" type="text" />', form_html)
#         self.assertNotIn('<input id="id_is_staff" name="is_staff" type="checkbox" />', form_html)
#         self.assertNotIn('<input checked="checked" id="id_is_active" name="is_active" type="checkbox" />', form_html)
#
#     def test_user_form_dupe_username(self):
#         LOGGER.debug("Test user form update")
#         data = {
#         'username': 'user1',
#         'password': 'user2password',
#         'email': 'user@user2.com',
#         'first_name': 'user',
#         'last_name': '2',
#         'last_login': '2012-09-04 06:00',
#         'date_joined': '2012-09-04 06:00',
#         'manually_edited': False
#         }
#         form = UserForm(data)
#         self.assertIn('already exists', form.errors['username'][0])
#
#     def test_user_form_update(self):
#         LOGGER.debug("Test user form create")
#         data = {
#         'username': 'user1',
#         'first_name': 'user',
#         'last_name': 'Two',
#         'last_login': '2012-09-04 06:00',
#         'date_joined': '2012-09-04 06:00',
#         'manually_edited': False
#         }
#         form = UserForm(data, instance=self.user1)
#         self.assertTrue(form.is_valid())
#         form.save()
#         self.assertEquals(self.user1.last_name, 'Two')



# Generated by Django 2.2.16 on 2020-11-17 11:09

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("socialprofile", "0007_auto_20200403_1525"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="proxysession",
            options={
                "verbose_name": "Monitor: sessione",
                "verbose_name_plural": "Monitor: sessioni",
            },
        ),
    ]
